/*
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import com.vividsolutions.jts.geom.Geometry;

import sim.engine.*;
import sim.display.*;
import sim.portrayal.grid.*;
import sim.portrayal.simple.*;
import java.awt.*;
import javax.swing.*;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import sim.display.Console;
import sim.display.Controller;
import sim.display.Display2D;
import sim.display.GUIState;


import java.io.*;
import java.util.*;
import java.util.ArrayList;

import sim.portrayal.network.*;
import sim.util.media.chart.*;
import sim.util.*;

import sim.portrayal.DrawInfo2D;

import sim.portrayal.geo.GeomVectorFieldPortrayal;
import sim.portrayal.geo.GeomPortrayal;
import sim.portrayal.simple.OvalPortrayal2D;
import sim.portrayal.simple.RectanglePortrayal2D;

import sim.util.geo.MasonGeometry;
import sim.util.geo.GeometryUtilities;

public class AmaSimWithUI extends GUIState
{
   private static final long serialVersionUID = 1L;
   public Display2D display;
   public JFrame displayFrame;

   public static String gfile = "";
   public static String oDir = "";
   public static String doA = "";


   //Geo Portrayals
   public GeomVectorFieldPortrayal homesPortrayal = new GeomVectorFieldPortrayal(true);
   public GeomVectorFieldPortrayal low_waterPortrayal = new GeomVectorFieldPortrayal(true);
   public GeomVectorFieldPortrayal high_waterPortrayal = new GeomVectorFieldPortrayal(true);
   public GeomVectorFieldPortrayal rioCutted_10mBufferPortrayal = new GeomVectorFieldPortrayal(true);


   SparseGridPortrayal2D householdPortrayal  = new SparseGridPortrayal2D();
   SparseGridPortrayal2D mosquitoPortrayal  = new SparseGridPortrayal2D();
   ObjectGridPortrayal2D geoGridPortrayal = new ObjectGridPortrayal2D();
   ObjectGridPortrayal2D elevationPortrayal = new ObjectGridPortrayal2D();

    public AmaSimWithUI() { super(new AmaSim(System.currentTimeMillis())); }
    public AmaSimWithUI(SimState state) { super(state); }

    // allow the user to inspect the model
    public Object getSimulationInspectedObject() { return state; }  // non-volatile

    public static String getName() { return "AmaSim"; }

   //====================================================
    public void init(Controller controller)
    {

        super.init(controller);
        AmaSim sim = (AmaSim)state;

        display = new Display2D(800,800,this); // at 400x400, we've got 4x4 per array position

        displayFrame = display.createFrame();
        controller.registerFrame(displayFrame);
        displayFrame.setVisible(true);
        display.setScale(0.85);

        //display.attach(homesPortrayal, "Homes");
        //display.attach(low_waterPortrayal, "Water");
        //display.attach(high_waterPortrayal, "Water");
        //display.attach(mosquitoPortrayal,"Mosquitos");


       //-To Chart-----------------------------------------


    }
 
 
   //====================================================
    public void setupPortrayals()
    {
        AmaSim sim = (AmaSim)state;

        homesPortrayal.setField(sim.homes);
        //homesLabelPortrayal b = new homesLabelPortrayal(new GeomPortrayal(Color.DARK_GRAY,true), Color.BLUE);
        //homesPortrayal.setPortrayalForAll(new OvalPortrayal2D(Color.BLUE, 3.0));
        homesPortrayal.setPortrayalForAll(new DrawHomesPortrayal() );
        //homesPortrayal.setPortrayalForAll(new GeomPortrayal(Color.CYAN,true));

        low_waterPortrayal.setField(sim.low_water);
        low_waterPortrayal.setPortrayalForAll(new GeomPortrayal(Color.BLUE,true));

        rioCutted_10mBufferPortrayal.setField(sim.rioCutted_10mBuffer);
        rioCutted_10mBufferPortrayal.setPortrayalForAll(new GeomPortrayal(Color.BLUE,true));

        high_waterPortrayal.setField(sim.high_water);
        high_waterPortrayal.setPortrayalForAll(new GeomPortrayal(Color.BLUE,true));
        //high_waterPortrayal.setPortrayalForAll(new HWPortrayal() );


        elevationPortrayal.setField(sim.elevationGrid);
        elevationPortrayal.setPortrayalForAll(new DrawHomesPortrayal() );

        //Mosquitos ---------------------------------------------
        mosquitoPortrayal.setField((sim.mosquitoGrid));
        DrawMosquitosPortrayal dmp = new DrawMosquitosPortrayal();
        mosquitoPortrayal.setPortrayalForAll( dmp );

        //Households---------------------------------------------
        householdPortrayal.setField((sim.householdGrid));
        householdPortrayal.setPortrayalForAll( new DrawHouseholdsPortrayal() );

        //Cover Layers ------------------------------------------
        //sim.readLayer = new InitializeLayers(state);
        //sim.readLayer.initAll(state);

        geoGridPortrayal.setField(sim.geoGrid);
        geoGridPortrayal.setPortrayalForAll( new DrawGeoLayersPortrayal() );

        display.reset();
        display.setBackdrop(Color.WHITE);
        display.repaint();
    }

    
   //====================================================
    public void start()
    {

        //System.out.println ("StarttttttttUIIIIIIIIIIIIIIII");
        final AmaSim sim = (AmaSim)state;
        sim.global_input_file = gfile;
        sim.outDir = oDir;
        sim.doAnalysis = doA;

        super.start();  // set up everything but replacing the display
        // set up our portrayals
        setupPortrayals();

        //display.attach(homesPortrayal, "Homes", -0.2, -0.9, true);
        //display.attach(waterPortrayal, "Water", -0.2, -0.9, true);

        display.attach(high_waterPortrayal, "high Water", 0, 0, true);
        //display.attach(elevationPortrayal, "elevation", 0, 0, true);
        
        display.attach(geoGridPortrayal,"GeoLayers");
        display.attach(householdPortrayal,"Households");
        display.attach(homesPortrayal, "Homes", 0, 0, true);
        display.attach(mosquitoPortrayal,"Mosquitos");
        display.attach(low_waterPortrayal, "low Water", 0, 0, true);
        display.attach(rioCutted_10mBufferPortrayal, "low Water", 0, 0, true);


        //final AmaSim sim = (AmaSim)state;

/*
        //---------------------------------------------------
        scheduleImmediateRepeat(true, new Steppable()
           {
              public void step(SimState state)
                 {

                    final AmaSim sim = (AmaSim)state;
                                                                            
                    int x = (int)state.schedule.time(); 
                                                 
                    // now add the data
                    if (x >= state.schedule.EPOCH && x < state.schedule.AFTER_SIMULATION & (x % 168 == 0)){

                    }
                  }
            });

        //To chart ================================
        //--------------------------------------------
*/

    }
            
   //====================================================
    public void load(SimState state)
    {
        super.load(state);
        // we now have new grids.  Set up the portrayals to reflect that
        setupPortrayals();
    }

       
   //====================================================
    public void quit()
    {
        super.quit();
        
        // disposing the displayFrame automatically calls quit() on the display,
        // so we don't need to do so ourselves here.
        if (displayFrame!=null) displayFrame.dispose();
        displayFrame = null;  // let gc
        display = null;       // let gc
    }



    //====================================================
    class DrawHomesPortrayal extends RectanglePortrayal2D
    {
        String what;
        Color  color;

        public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
        {
               if(object == null){return;}
               color = new Color(100, 100, 100);
               //System.out.println ("Homes draw");

               Rectangle2D.Double draw = info.draw;
               scale = 0.3;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int x = (int) (draw.x - width / 2.0);
               final int y = (int) (draw.y - height / 2.0);
               //System.out.println (x + " " + y);
               //
               final int w = (int) (width);
               final int h = (int) (height);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

               //System.exit(0);
        }
    }

   //====================================================
    public static void main(String[] args)
    {
       new AmaSimWithUI().createController();

       gfile = args[0];
       oDir  = args[1];

       if(args.length > 2)
       {
          if(args[2].equals("doAnalysis"))doA = args[2];
       }

       AmaSimWithUI simWithUI = new AmaSimWithUI();
       //Console console = new Console(simWithUI);
       //console.setVisible(true);
       //System.out.println ("MainUI");
    }

    //====================================================
    class DrawMosquitosPortrayal extends OvalPortrayal2D
    {
        Color  color;

        public DrawMosquitosPortrayal()
        {
        
        }

        public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
        {
            Mosquito mos = (Mosquito)object;

            double infected = mos.getInfected();
            boolean infectious = mos.getInfectious();

            //if(infected > 0)
            //{
            //   paint = new Color(180, 0, 0);
            //}
            //else if(infectious)
            //{
            //   paint = new Color(240, 0, 0);
            //}
/*
            if(mos.getBeforeMating())
            {
               paint = new Color(0, 255, 0);
            }
            else if(mos.getHostSearch())
            {
               paint = new Color(255, 255, 0);
            }
            else if(mos.getBreedingHabitatSearch())
            {
               paint = new Color(0, 0, 255);
            }
            */

            if(mos.getNoNulliparous())
            {
               paint = new Color(255, 0, 0);
            }
            else
            {
               paint = new Color(0, 0, 0);
            }
            scale = 0.7;
            super.draw(object, graphics, info);


        }

    }

    //====================================================
    class DrawHouseholdsPortrayal extends RectanglePortrayal2D
    {
        String what;
        Color  color;


        public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
        {
            //System.out.println ("Hoseholsss");
            Household hh  = (Household)object;

            double incidence = 0;

            if(hh.getWeeksCounter() != 0)
            {
               incidence = hh.getAvgMalariaIncidence() / hh.getWeeksCounter();
            }
            if(incidence <= 0)incidence = 0;
            if(incidence >= 255)incidence = 255;
            if(incidence > 0.00001)incidence = 1.0;
            //System.out.println ("Hoseholsss incidence : " + incidence);

            int xcor = hh.getXpos();
            int ycor = hh.getYpos();

            int col = (int)(255 * incidence);

            //System.out.println ("col " + col);

            color = new Color(col, 0, 0);

            scale = 1.0;

            Rectangle2D.Double draw = info.draw;
            final double width = draw.width * scale;
            final double height = draw.height * scale;

            final int w = (int) (width/1.0);
            final int h = (int) (height/1.0);

            final int x = (int) (draw.x - w / 1.0);
            final int y = (int) (draw.y - h / 1.0);

            graphics.setColor(color);

            graphics.fillRect(x, y, w, h);

        }
    }

    //====================================================
    class DrawGeoLayersPortrayal extends RectanglePortrayal2D
    {
        String what;
        Color  color;

        public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
        {
            CoverPixel pixel = (CoverPixel)object;

            int isWater = pixel.getIsWater();
            int isRiverBank = pixel.getIsRiverBank();

            int xcor = pixel.getXcor();
            int ycor = pixel.getYcor();

            scale = 0.80;

            boolean belev = true;

            if(belev)
            {
               int elev = (int)Math.round(pixel.getElevation());
               int maxElev = 106;
               int minElev = 79;
               elev     = (int)(255 * (elev  - minElev)/(maxElev - minElev));
               if(elev > 255){elev = 255;}
               if(elev < 0){elev = 0;}


               //System.out.println (elev);
               color = new Color(elev, 0, 255 - elev);
               //color = new Color(255, 255, 255);

               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);
            }


            if(pixel.getIsRiverBankHigh() == 1)
            {
               color = new Color(255, 255, 255);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }

            if(pixel.getIsRiverBankLow() == 1)
            {
               color = new Color(0, 255, 0);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }
            
            if(pixel.getIsWater() == 1)
            {
               color = new Color(0, 0, 255);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }

 
            if(pixel.getIsWater() == 0 && pixel.getIsRiverBank() == 0)
            {
               color = new Color(0, 255, 0);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }

 



            if(pixel.getIsRiverBank() == 1)
            {
               color = new Color(0, 0, 0);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }

 
            if(pixel.getSpotify() == 1)
            {
               color = new Color(255, 0, 0);


               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }


            if(pixel.getIsBufferNoMos())
            {
               color = new Color(0, 0, 0);

               //System.out.println ("Buffer!");

               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);

            }
 


            /*if(pixel.getSpotify()  == 1)
            {
               //return;
               color = new Color(255, 0, 255);
               //color = new Color(255, 255, 255);

               Rectangle2D.Double draw = info.draw;
               final double width = draw.width * scale;
               final double height = draw.height * scale;
   
               final int w = (int) (width/1.0);
               final int h = (int) (height/1.0);
   
               final int x = (int) (draw.x - w / 2.0);
               final int y = (int) (draw.y - h / 2.0);
   
               graphics.setColor(color);
   
               graphics.fillRect(x, y, w, h);
            }*/




        }
    }

    //====================================================
    class HWPortrayal extends GeomPortrayal
    {
        String what;
        Color  color;

        public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
        {

            //System.out.println ("Hoseholsss");
            MasonGeometry mg  = (MasonGeometry)object;
            what = mg.getStringAttribute("CUBIERTA");
            System.out.println (what);

            if(what.equals("AGUA"))
            {
               color = new Color(0, 0 ,255);
   
               graphics.setColor(color);
            }

        }
    }


}

