/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;
import java.util.ArrayList;
import java.util.List;

import sim.util.geo.MasonGeometry;

public class CoverPixel implements Steppable
{
   private static final long serialVersionUID = 1L;

   int xcor        = 0;
   int ycor        = 0;

   public AmaSim sim;

   public int getXcor(){return xcor;};
   public int getYcor(){return ycor;};

   int isWater = 0;
   public int  getIsWater(){return isWater;};
   public void setIsWater(int w)
   {
       if(isWater == 0 && w == 1)
       {
           setIsProductiveTimer(((int)Math.round(sim.aquaticStageTime))); 
       }

       isWater = w;
   }

   int isWaterHigh = 0;
   public int  getIsWaterHigh(){return isWaterHigh;};
   public void setIsWaterHigh(int w){isWaterHigh = w;};

   int isRiverExcluded = 0;
   public int  getIsRiverExcluded(){return isRiverExcluded;};
   public void setIsRiverExcluded(int w){isRiverExcluded = w;};

   int isWaterLow = 0;
   public int  getIsWaterLow(){return isWaterLow;};
   public void setIsWaterLow(int w){isWaterLow = w;};

   int isRiverBank = 0;
   public int  getIsRiverBank(){return isRiverBank;};
   public void setIsRiverBank(int w)
   {
       if(w == 1)
       {
          if(isProductiveTimer < 0)
          {
             setIsProductiveTimer(((int)Math.round(sim.aquaticStageTime))); 
          }
       }

       isRiverBank = w;
   }

   int isDryHigh = 0;
   public int  getIsDryHigh(){return isDryHigh;};
   public void setIsDryHigh(int w){isDryHigh = w;};

   int isDone = 0;
   public int  getIsDone(){return isDone;};
   public void setIsDone(int w){isDone = w;};

   public MasonGeometry masonGeometry = null;
   public void setMasonGeometry(MasonGeometry mg){masonGeometry = mg;};
   public MasonGeometry getMasonGeometry(){return masonGeometry;};

   public Bag linkPixels = new Bag();
   public void setLinkPixels(Bag mg){linkPixels = mg;};
   public Bag getLinkPixels(){return linkPixels;};

   public String type = null;
   public void setType(String mg){type = mg;};
   public String getType(){return type;};

   public double riverBankLength = 0.0;
   public void setRiverBankLength(double mg){riverBankLength = mg;};
   public double getRiverBankLength(){return riverBankLength;};

   public double riverBankLengthHigh = 0.0;
   public void setRiverBankLengthHigh(double mg){riverBankLengthHigh = mg;};
   public double getRiverBankLengthHigh(){return riverBankLengthHigh;};

   public double riverBankLengthLow = 0.0;
   public void setRiverBankLengthLow(double mg){riverBankLengthLow = mg;};
   public double getRiverBankLengthLow(){return riverBankLengthLow;};

   public double spotify = 0.0;
   public void setSpotify(double mg){spotify = mg;};
   public double getSpotify(){return spotify;};

   public double elevation = 0.0;
   public void setElevation(double mg){elevation = mg;};
   public double getElevation(){return elevation;};

   public boolean containsHousehold = false;
   public void setContainsHousehold(boolean mg){containsHousehold = mg;};
   public boolean getContainsHousehold(){return containsHousehold;};

   public Bag households = new Bag();
   public void setHouseholds(Bag mg){households = mg;};
   public Bag getHouseholds(){return households;};

   //this is used to build the mosquitos pixel grid -----
   public boolean containsBreeding = false;
   public void setContainsBreeding(boolean mg){containsBreeding = mg;};

   int isRiverBankHigh = 0;
   public int  getIsRiverBankHigh(){return isRiverBankHigh;};
   public void setIsRiverBankHigh(int w){isRiverBankHigh = w;};

   int isRiverBankLow = 0;
   public int  getIsRiverBankLow(){return isRiverBankLow;};
   public void setIsRiverBankLow(int w){isRiverBankLow = w;};

   int isProductiveTimer = -300;
   public int  getIsProductiveTimer(){return isProductiveTimer;};
   public void setIsProductiveTimer(int w){isProductiveTimer = w;};

   boolean isProductive = false;
   public boolean getIsProductive(){return isProductive;};
   public void setIsProductive(boolean w){isProductive = w;};

   boolean isBufferNoMos = false;
   public boolean getIsBufferNoMos(){return isBufferNoMos;};
   public void setIsBufferNoMos(boolean w){isBufferNoMos = w;};

   public Stoppable stopper;

   public double mosRest = 0.0;
   public void setMosRest(double mg){mosRest = mg;};
   public double getMosRest(){return mosRest;};

   public List<Integer> eggsOvoposited = new ArrayList<Integer>();
   public List<Integer> getEggsOvoposited(){return eggsOvoposited;}; 
   public void setEggsOvoposited(int nEggs)
   {
       //int len = (int)(sim.aquaticStageTime / 24);
       eggsOvoposited.remove(0);
       //eggsOvoposited.set((len - 1), nEggs);
       eggsOvoposited.add(nEggs);
   }

   int nEggs = 0;
   public int  getNEggs(){return nEggs;};
   public void setNEggs(int w){nEggs = w;};
   public void addEgg(){nEggs++;};

   //====================================================
   public CoverPixel(final SimState state, int Pxcor, int Pycor)
   {

      sim = (AmaSim)state;
      xcor = Pxcor;
      ycor = Pycor;

      isWater     = 0;
      isRiverBank = 0;
      elevation   = 0.0;
      isDryHigh   = 0;

      this.stopper = sim.schedule.scheduleRepeating(this);

      //Initialize ovoposited eggs
      int len = (int)(sim.aquaticStageTime / 24);
      for(int i = 0;  i < (len - 1); i++)
      {
         eggsOvoposited.add(0);
      }

   }

   //====================================================
   public void step(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;

      isProductiveTimer--;
      if(isWater == 1)
      {
         if(isProductiveTimer == 0) isProductive = true; 
         else if(isProductiveTimer > 0 ) isProductive = false; 
      }
      if(isWater == 0)
      {
          isProductive = false; 
          isProductiveTimer = -300;
      }
   }
   
   //====================================================
   public Bag getVonNeumannNeighbors(final SimState state, String grid)
   {
      final AmaSim sim = (AmaSim)state;
      //VanNeumann neighbors-----------------------
      Integer coors[] = {0, 0};
      Bag neig = new Bag();
      CoverPixel tcp;

      //------------------------------------------
      coors[0] = xcor + 1;
      coors[1] = ycor;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);


      //------------------------------------------
      coors[0] = xcor - 1;
      coors[1] = ycor;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);


      //------------------------------------------
      coors[0] = xcor;
      coors[1] = ycor + 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor;
      coors[1] = ycor - 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      return neig;

   }


   //====================================================
   public Bag getMooreNeighbors(final SimState state, String shuffle, String grid)
   {
      final AmaSim sim = (AmaSim)state;
      //VanNeumann neighbors-----------------------
      Integer coors[] = {0, 0};
      Bag neig = new Bag();
      CoverPixel tcp;

      //------------------------------------------
      coors[0] = xcor + 1;
      coors[1] = ycor;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor - 1;
      coors[1] = ycor;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);


      //------------------------------------------
      coors[0] = xcor;
      coors[1] = ycor + 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor;
      coors[1] = ycor - 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor + 1;
      coors[1] = ycor + 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor + 1;
      coors[1] = ycor - 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor - 1;
      coors[1] = ycor + 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      //------------------------------------------
      coors[0] = xcor - 1;
      coors[1] = ycor - 1;
      coors = sim.utils.pbc(state, coors, "toroidals", grid);
      tcp = sim.utils.getCoverPixelFromCoords(state, coors[0], coors[1], grid);
      neig.add(tcp);

      if(shuffle.equals("doShuffle"))
      {
          neig.shuffle(state.random);
      }

      return neig;

   }

   //====================================================
   public boolean pixelContainsHouseholds(SimState state)
   {
      final AmaSim sim = (AmaSim)state;
      Bag households = sim.householdGrid.getObjectsAtLocation(
              this.getXcor(), this.getYcor()
              );

      if(households != null && households.size() > 0)return true;
      else return false;
   }


   //====================================================
   public CoverPixel getContainsBreeding(SimState state)
   {
      //input is a mosquito pixel 
      Bag linkMCp = this.getLinkPixels();

      linkMCp.shuffle(state.random);

      for(int i =  0; i < linkMCp.size(); i++)
      {
          CoverPixel gcp = (CoverPixel)linkMCp.objs[i];

          if(gcp.getIsWaterHigh() == 0) continue;
          if(gcp.getIsRiverExcluded() == 1) continue;

          //if(gcp.getIsWater() == 1 && gcp.getIsWaterLow() == 0)
          if(gcp.getIsWater() == 1) return gcp;
      }
      return null;
   
   }



//============================================================   
}
