/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import java.io.*;
import java.util.*;
import java.util.ArrayList;

import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.WorkbookFactory; // This is included in poi-ooxml-3.6-20091214.jar
import org.apache.poi.ss.usermodel.Workbook;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;

import java.util.logging.Level;
import java.util.logging.Logger;
import sim.engine.SimState;
import sim.util.*;

import java.util.List;
import java.util.ArrayList;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.poi.ss.usermodel.DateUtil;

public class DoAnalysis
{
    //private static final long serialVersionUID = -4554882816749973618L;
    private static final long serialVersionUID = 1L;

    boolean first = true;

    public String file_name;
    public List<String> titles = new ArrayList<String>();
    public List<List<Object>> data = new ArrayList<List<Object>>();

    public String outFile = "";

    //====================================================
    public DoAnalysis(AmaSim sim)
    {
       outFile = sim.outPath + "avgData_" + sim.outDir  +  ".xls";
    }

    //====================================================
    public void readFile(AmaSim sim, String rFile)
    {
      int stats  = 0;
      int numRow = 0;
 

      rFile = sim.outPath + rFile;
      System.out.println (rFile);
 
      try{
             Workbook workbook = WorkbookFactory.create(new FileInputStream(rFile) );
  
             Sheet sheet = workbook.getSheet(sim.outDir);

             List<Object> rData   = new ArrayList<Object>();
             List<List<Object>> dataF   = new ArrayList<List<Object>>();
             titles = new ArrayList<String>();
  
             for(Row row : sheet)
             { 
               numRow++;
               rData   = new ArrayList<Object>();
               stats = 0;
               for(Cell cell : row)
               { 
                 stats++;

                 //String string = Integer.toString((int)cell.getNumericCellValue());
                 
                 //System.out.println ("-----------------------");
                 //System.out.println (cell.getCellType());
                 //System.exit(0);

                 if(numRow == 1)
                 {
                    titles.add(cell.getRichStringCellValue().getString() ); 
                    continue;
                 }
                 
                 switch (cell.getCellType()) {
                     case Cell.CELL_TYPE_STRING:
                         rData.add(cell.getRichStringCellValue().getString() );
                         break;
                     case Cell.CELL_TYPE_NUMERIC:
                         rData.add((Object)cell.getNumericCellValue());
                         break;
                     default:
                         rData.add(0.0);
                         //System.out.println ("Error cell value:"  + cell.getCellType());
                     
                 //System.exit(0);
                 }
               }

               //if(stats != 30)
               //{
               //   System.out.println ("Num col: " + stats);
               //}
               dataF.add(rData);
 
             }

             if(first)
             {

                int len = dataF.size();

                for(int i = 0; i < len; i++)
                {
                    data.add(dataF.get(i)); 
                }

             }
             else
             {
                //System.out.println("---------");
                sumAverages(sim, dataF); 
             }

             //System.exit(0);
  
  
  
          }
          catch(FileNotFoundException e)
              {
                 System.out.println(e);
              }
          catch(IOException e)
              {
                 System.out.println(e);
              }
          catch(InvalidFormatException e)
              {
                 System.out.println(e);
              }
 
 
    }

    //====================================================
    public void calcAverages(AmaSim sim, int numFiles)
    {
        List<Object> dataLine = new ArrayList<Object>();
        List<Object> line_tmp = new ArrayList<Object>();
        List<List<Object>> data_tmp = new ArrayList<List<Object>>();

        Object obj_tmp;
        Object objData;

        double d_data = 0;

        Object obj;
       
        int len = data.size();
    
        for(int i = 0; i < len; i++)
        {
            dataLine = data.get(i);
            line_tmp = new ArrayList<Object>();

            int lenLine = dataLine.size();
            //System.out.println(lenLine);

            for(int j = 0; j < lenLine; j++)
            {
               objData = (Object)dataLine.get(j); 
               obj_tmp = new Object();

               if(objData instanceof String)
               {
                  obj_tmp = objData; 
               }
               else if(objData instanceof Double)
               {
                  d_data = (Double)objData;
                  d_data = d_data / numFiles;
                  obj_tmp = (Object)d_data;
               }

               line_tmp.add(obj_tmp);

               //System.out.println(obj);
               //System.exit(0);
            }
            //System.exit(0);

            data_tmp.add(line_tmp);
        }

        data = data_tmp;
    }


    //====================================================
    public void sumAverages(AmaSim sim, List<List<Object>> list)
    {
        List<Object> readLine = new ArrayList<Object>();
        List<Object> dataLine = new ArrayList<Object>();
        List<Object> line_tmp = new ArrayList<Object>();
        List<List<Object>> data_tmp = new ArrayList<List<Object>>();

        Object obj_tmp;
        Object objRead;
        Object objData;

        double d_read = 0;
        double d_data = 0;

        Object obj;
       
        int len = list.size();
    
        for(int i = 0; i < len; i++)
        {
            dataLine = data.get(i);
            readLine = list.get(i);
            line_tmp = new ArrayList<Object>();

            int readLinen = readLine.size();
            int dataLinen = dataLine.size();
            //if(dataLinen != 30 || readLinen != 30)
            //{
            //   System.out.println("dataLine: " + dataLinen);
            //   System.out.println("readLine: " + readLinen);
            //}


            int lenLine = readLinen;
            for(int j = 0; j < lenLine; j++)
            {
               objData = (Object)dataLine.get(j); 
               objRead = (Object)readLine.get(j); 
               obj_tmp = new Object();

               if(objData instanceof String)
               {
                  obj_tmp = objData; 
               }
               else if(objData instanceof Double)
               {
                  d_read = (Double)objRead;
                  d_data = (Double)objData;
                  d_data = d_data + d_read;
                  obj_tmp = (Object)d_data;
               }


               line_tmp.add(obj_tmp);

               //System.out.println(obj);
               //System.exit(0);
            }
            //System.exit(0);

            data_tmp.add(line_tmp);
        }

        data = new ArrayList<List<Object>>();
        data = data_tmp;

    }

 
    //====================================================
    public void read(AmaSim sim)
    {
      System.out.println("================================================");
      System.out.println ("Analyzing the outputs  ........................");
      System.out.println(" ");
 
      File theDir = new File(sim.outPath);
      String [] directoryContents = theDir.list();
 
      int numFiles = 0;
 
      for(String fileName: directoryContents) 
      {
          System.out.println (fileName);
          String delims = "\\.";
          String[] words = fileName.split(delims);

          //System.out.println (words.length);
 
          if(words.length < 2)continue;
          if(!words[1].equals("xls"))continue;
 
          delims = "_";
          words = words[0].split(delims);
 
          int len = words.length;
 
          if(!words[0].equals("output"))continue;
 
          numFiles++;
          readFile(sim, fileName);
 
          System.out.println ("File number " + numFiles + " done ...........");

          first = false;
      }

      calcAverages(sim, numFiles); 

      writeInput(sim);
      
      try {
             writeToFile(sim);
              
         } catch (FileNotFoundException e) {
             e.printStackTrace();
         } catch (IOException e) {
             e.printStackTrace();
         }

 
      System.out.println(" ");
    }

    //====================================================
    public void writeInput(AmaSim sim)
    {
      ReadInput input = new ReadInput(sim);
      List<String> list = input.getInputList();

      System.out.println("================================================");
      System.out.println ("Writing  the input file into the averages file.");
      System.out.println(" ");

      HSSFWorkbook workbook = new HSSFWorkbook();
      HSSFSheet sheet = workbook.createSheet("Input File");
 
      int rownum = 0;
      int size = list.size();

      for(int i = 0; i < size; i++)
      {
          rownum++;
          Row row = sheet.createRow(rownum);

          String line = list.get(i);

          Cell cell = row.createCell(0);
          cell.setCellValue((String)line);

      }
 
      try {

          FileOutputStream out = 
             new FileOutputStream(new File(outFile));
          workbook.write(out);
          out.close();
          System.out.println("Excel written successfully..");
           
      } catch (FileNotFoundException e) {
          e.printStackTrace();
      } catch (IOException e) {
          e.printStackTrace();
      }

      //System.exit(0);

    }



    //====================================================
    public void writeToFile(AmaSim sim) throws IOException
    {
       System.out.println("================================================");
       System.out.println ("Writing  averages .............................");
       System.out.println(" ");
 
       FileInputStream file = new FileInputStream(outFile);
 
       HSSFWorkbook workbook = new HSSFWorkbook(file);
       HSSFSheet sheet;
 
       sheet = workbook.getSheet(sim.outDir);
       //If the sheet !exists a new sheet is created --------- 
       if(sheet == null)sheet = workbook.createSheet(sim.outDir);
 
       workbook.setSheetOrder(sim.outDir, 0);
 
       Cell cell = null;
 
       int lastRow = sheet.getLastRowNum();
       int cellnum = 0;
       //lastRow++;

       List<Object> line_tmp = new ArrayList<Object>();
 
       //System.out.println("Last row:" + lastRow);
       //lastRow++;
 

       //writes titles ----------------------------------
       int size = titles.size();
       Row row = sheet.createRow(lastRow++);
       cellnum = 0;
 
       for(int i = 0; i < size; i++)
       {
           String title = titles.get(i);
           
           cell = row.createCell(cellnum++);

           cell.setCellType(1);
           cell.setCellValue(title);
       }

       
       size = data.size();

       for(int i = 1; i < size; i++)
       {

           cellnum = 0;
           row = sheet.createRow(lastRow++);
 
           line_tmp = data.get(i);

           //System.out.println("------------");
           //System.out.println(line_tmp.size());
           //System.exit(0);
 
           for (Object obj : line_tmp) 
           {
              //System.out.println("------------");
              cell = row.createCell(cellnum++);
 
              if(obj instanceof String)
              {
                 cell.setCellType(1);
                 cell.setCellValue((String)obj);
                 //System.out.println(obj);
                 //System.out.println("String....");
              }
              else if(obj instanceof Double)
              {
                 cell.setCellType(0);
                 cell.setCellValue((Double)obj);
                 //System.out.println("Double....");
              }
           }
       }
       //System.exit(0);
 
       file.close();
  
       File out_file = new File(outFile);
 
       FileOutputStream out = new FileOutputStream(out_file);
 
       workbook.write(out);
       out.close();
       System.out.println("Analysis data written successfully.....");

       


    }
 

}
