/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;
import java.util.*; 

//----------------------------------------------------
public class Human extends Animal 
{
   private static final long serialVersionUID = 1L;

   AmaSim sim = null;

   Household household = null;
   public Household getHousehold(){return household;};
   public void setHousehold(Household y){household = y;};

   public boolean hInfected = false;
   public boolean getHInfected(){return hInfected;};
   public void setHInfected(boolean y){hInfected = y;};

   public boolean hInfectious = false;
   public boolean getHInfectious(){return hInfectious;};
   public void setHInfectious(boolean y){hInfectious = y;};

   public boolean treated = false;
   public boolean getTreated(){return treated;};
   public void setTreated(boolean y){treated = y;};

   public boolean malariaInOneYear = false;
   public boolean getMalariaInOneYear(){return malariaInOneYear;};
   public void setMalariaInOneYear(boolean y){malariaInOneYear = y;};

   public int malariaInOneYearTimer = -100;
   public int getMalariaInOneYearTimer(){return malariaInOneYearTimer;};
   public void setMalariaInOneYearTimer(int y){malariaInOneYearTimer = y;};

   public boolean susceptible = true;
   public boolean getSusceptible(){return susceptible;};
   public void setSusceptible(boolean y){susceptible = y;};

   public int infectiousPeriod = -100;
   public Integer getInfectiousPeriod(){return infectiousPeriod;};
   public void setInfectiousPeriod(int y){infectiousPeriod = y;};

   public int infectionTimer = -100;
   public Integer getInfectionTimer(){return infectionTimer;};
   public void setInfectionTimer(int y){infectionTimer = y;};

   public int asymptomatic = 0;
   public Integer getAsymptomatic(){return asymptomatic;};
   public void setAsymptomatic(int y){asymptomatic = y;};

   HashMap<String, Double> asymptomaticity = new HashMap<String, Double>();
   public HashMap getAsymptomaticity(){return asymptomaticity;};
   public void setAsymptomaticity(HashMap y){asymptomaticity = y;};

   public CoverPixel cpPosition = null;
   public void setCpPosition(CoverPixel mg){cpPosition = mg;};
   public CoverPixel getCpPosition(){return cpPosition;};

   public int timeToSymptoms = -100;
   public Integer getTimeToSymptoms(){return timeToSymptoms;};
   public void setTimeToSymptoms(int y){timeToSymptoms = y;};

   public int timeToCommunicability = -100;
   public Integer getTimeToCommunicability(){return timeToCommunicability;};
   public void setTimeToCommunicability(int y){timeToCommunicability = y;};

   public int timeToRecover = -100;
   public Integer getTimeToRecover(){return timeToRecover;};
   public void setTimeToRecover(int y){timeToRecover = y;};

   public int identity = 0;
   public Integer getIdentity(){return identity;};
   public void setIdentity(int y){identity = y;};

   public double probFail = 0;
   public double getProbFail(){return probFail;};
   public void setProbFail(double y){probFail = y;};

   public boolean gotMalaria = false;
   public boolean getGotMalaria(){return gotMalaria;};
   public void setGotMalaria(boolean y){gotMalaria = y;};

   public int numMaxProtection = -40000;

   public boolean vivaxRecurrence = false;
   public boolean getVivaxRecurrence(){return vivaxRecurrence;};
   public void setVivaxRecurrence(boolean y){vivaxRecurrence = y;};

   public int vivaxRecurrenceTime = -100;
   public int getVivaxRecurrenceTime(){return vivaxRecurrenceTime;};
   public void setVivaxRecurrenceTime(int y){vivaxRecurrenceTime = y;};

   public boolean isProtected = false;
   public boolean getIsProtected(){return isProtected;};
   public void setIsProtected(boolean y){isProtected = y;};

   //====================================================
   public Human(SimState state, CoverPixel cp, Household ph)
   {
       super(state, cp);
       cpPosition = cp;
       sim = (AmaSim)state;
       //this.setAge(0);

       this.setSpecies("Homo Sapiens");

       household = ph;

       household.addHuman(this);

       this.init(state);

       this.stopper = sim.schedule.scheduleRepeating(this);

       sim.human_bag.add(this);

       asymptomaticity.put("Falciparum", new Double(0.0));
       asymptomaticity.put("Vivax", new Double(0.0));
   
       identity = sim.hIdent;
       sim.hIdent++;

       //double ran = state.random.nextDouble();
       //if(ran <= sim.fractProtectedPeople)setIsProtected(true);

   }

   //====================================================
   public void init(SimState state)
   {
       sim = (AmaSim)state;

       this.setAge(state.random.nextInt(70) * 8640);
      
       double ran = state.random.nextDouble();

       if(setAsymptomaticity(state))return;

       if(state.random.nextInt(15) == 0)
       {
          susceptible = false;

          ran = state.random.nextDouble();

          if(ran > 0.5)plasmodium = sim.Falciparum;
          else plasmodium = sim.Vivax;

          hInfected = true;

          infectionTimer = 0;
    
          ran = state.random.nextDouble();
    
          if(ran > sim.fractHumanTreated)treated = false;
          else treated = true;
    
          if(!treated)timeToRecover = 2 + state.random.nextInt(plasmodium.getHumanInfectiousPeriodNotTreated());           
    
          int numDays = plasmodium.getIntrinsicIncubation().get(1) - plasmodium.getIntrinsicIncubation().get(0);
          int ranI = state.random.nextInt(numDays + 1);
          ranI = ranI + plasmodium.getIntrinsicIncubation().get(0);
          timeToSymptoms = state.random.nextInt(ranI + 2);
          //System.out.println ("Time to symptoms: " + timeToSymptoms/24);


          timeToCommunicability = 2 + state.random.nextInt(plasmodium.getStartCommunicability());

       
       }

   }


   //====================================================
   public void step(SimState state)
   {
       //System.out.println ("-----------------------------");
       //System.out.println (this.getAge() + "  mosquito age");
       super.step(state);
       sim = (AmaSim)state;

       if(hInfected)infectionDynamics(state);

       //See if hte mosquito have to die -----------------
       this.setAge(this.getAge() + 1);
       if(this.getAge() > 80 * 8640)
       {
          this.stopper.stop();         
          sim.human_bag.remove(this);
          household.removeHuman(this);

          return;
       }

       //Upgrade asymptmaticity: every hour a little fraction is lost
       double as;
       as = asymptomaticity.get("Falciparum");
       as = as - sim.Falciparum.getFractAsymptomaticityLostPerHour();
       if(as < 0.0) as = 0.0;
       asymptomaticity.put("Falciparum", as);

       as = asymptomaticity.get("Vivax");
       as = as - sim.Vivax.getFractAsymptomaticityLostPerHour();
       if(as < 0.0) as = 0.0;
       asymptomaticity.put("Vivax", as);

       //Incubation period ends -> Infectious period starts --------------
       if(infected == 0)
       {
           //System.out.println (this.getSpecies() + "  become infectious!");
           this.setInfectiousPeriod(plasmodium.getHumanInfectiousPeriodNotTreated());
           this.setInfectious(true);
       }

       //Upgrade timers----------------------------------
       infectiousPeriod--;
       malariaInOneYearTimer--;
       vivaxRecurrenceTime--;

       //The humnan is no more infectiuos: recovered state -> is now susceptible
       if(infectiousPeriod == 0)
       {
           //human gain a fract of asymptomaticity---------------------
           as = asymptomaticity.get(plasmodium.getType());
           as = as +  plasmodium.getFractAsymptomaticityGainedPerInfection();
           if(as < 0.0)as = 0.0;
           asymptomaticity.put(plasmodium.getType(), as);

           setInfectious(false);
           setPlasmodium(null);
       }

       infected--;

       if( malariaInOneYearTimer == 0 )setMIOYOff();
       if( sim.HmonthlyIncidence <= sim.setProtOff )setMIOYOff();

       //fact = 0 all full period
       //fact = 1 scaling all the time
       //double fact = sim.protectionPeriodFact;
       //if(malariaInOneYearTimer > fact * sim.malariaProtection)
       //{
       //   probFail = sim.probFailMalariaProtection;
       //}
       //else
       //{
       //   probFail = sim.probFailMalariaProtection * (double)( malariaInOneYearTimer / (sim.malariaProtection * fact)  );
       //}

       probFail = sim.probFailMalariaProtection;

       if( malariaInOneYearTimer < 0 && malariaInOneYear )
       {
          System.out.println ("error in Human malariainoneyear   !!!!-------");
          System.exit(0);
       }

       if(vivaxRecurrence && vivaxRecurrenceTime == 0)setupRecurrence(state);
   }


   //====================================================
   public void setupInfectionAfterBite(SimState state, Mosquito mosquito)
   {
      double ran = state.random.nextDouble();

      if(mosquito != null && mosquito.getPlasmodium() != null)
      {
         if(ran < sim.vivaxRiskRecurrence && mosquito.getPlasmodium().getType().equals("Vivax")) 
         {
            vivaxRecurrence = true;   
   
            ran = state.random.nextDouble();
            ran = ran - 0.5;
            ran = ran / 2.5;
            vivaxRecurrenceTime = (int)Math.round(sim.vivaxTimeRecurrence * (1 + ran));   
         }
      }
      
      sim = (AmaSim)state;

      gotMalaria = true;
      household.setGotMalaria(true);

      susceptible = false;
      if(mosquito != null && mosquito.getPlasmodium() != null)
      {
         setPlasmodium(mosquito.getPlasmodium());
      }
      else
      {
         plasmodium = sim.Vivax;
      }

      hInfected = true;
      infectionTimer = 0;

      ran = state.random.nextDouble();

      if(ran > sim.fractHumanTreated)treated = false;
      else treated = true;

      if(!treated)
      {
          
         ran = state.random.nextDouble();
         ran = ran - 0.5;
         ran = ran/2.5;
         timeToRecover = plasmodium.getHumanInfectiousPeriodNotTreated();           
         timeToRecover = timeToRecover + (int)Math.round(ran * timeToRecover);
      }

      
      //ran = state.random.nextDouble();
      //System.out.println ("-------------------------");
      int numDays = plasmodium.getIntrinsicIncubation().get(1) - plasmodium.getIntrinsicIncubation().get(0);
      //System.out.println ("Num days: " + numDays/24);
      int ranI = state.random.nextInt(numDays + 1);
      //System.out.println ("Num days: " + ranI/24);
      ranI = ranI + plasmodium.getIntrinsicIncubation().get(0);
      timeToSymptoms = ranI;
      //System.out.println ("Plasm: " + plasmodium.getType() + "000000000000000000000000000000000000000");
      //System.out.println ("Time to symptoms: " + timeToSymptoms);
      //System.out.println ("-------------------------");

      sim.weeklyAvgTimeToSymptoms = sim.weeklyAvgTimeToSymptoms + timeToSymptoms;
      sim.weeklyAvgTimeToSymptomsCounter++;

      ran = state.random.nextDouble();
      ran = ran - 0.5;
      ran = ran/2.5;
      timeToCommunicability = plasmodium.getStartCommunicability();
      timeToCommunicability = timeToCommunicability + (int)Math.round(ran * timeToCommunicability);

      if(numMaxProtection > 0)
      {
         setMIOYOn();
         malariaInOneYeaToHousehold(state);
      }

   }

   //====================================================
   public void setMIOYOff()
   {
      malariaInOneYear = false;
      malariaInOneYearTimer = -150;
      //System.exit(0);
   }

   //====================================================
   public void setMIOYOn()
   {
       //System.out.println ("biten!!!!!!!-------");
       //System.out.println ("Num Mlaria in one year: " + getNM(sim));
       //System.out.println ("Num people: " + getP(sim));
       malariaInOneYear = true;
       malariaInOneYearTimer = sim.malariaProtection;
       //System.out.println ("Mal in one year timer: " + malariaInOneYearTimer);
   }

   //====================================================
   public void infectionDynamics(SimState state)
   {
       infectionTimer++;

       if(timeToSymptoms == infectionTimer && treated)
       {
          double ran = state.random.nextDouble();
          ran = ran - 0.5;
          ran = ran/2.5;
          timeToRecover = plasmodium.getHumanInfectiousPeriodTreated();           
          timeToRecover = timeToRecover + infectionTimer + (int)Math.round(ran * plasmodium.getHumanInfectiousPeriodTreated());           

          //Pitxi era cosi
          //double ran = state.random.nextDouble();
          //ran = ran - 0.5;
          //ran = ran/2.5;
          //timeToRecover = infectionTimer + (int)Math.round(ran * plasmodium.getHumanInfectiousPeriodTreated());           

          sim.weeklyMalariaCases++;
          if(plasmodium.getType().equals("Falciparum"))sim.weeklyMalariaCasesFalciparum++;
          if(plasmodium.getType().equals("Vivax"))sim.weeklyMalariaCasesVivax++;

          sim.monthlyMalariaCases++;
          if(plasmodium.getType().equals("Falciparum"))sim.monthlyMalariaCasesFalciparum++;
          if(plasmodium.getType().equals("Vivax"))sim.monthlyMalariaCasesVivax++;

          sim.yearlyMalariaCases++;
          if(plasmodium.getType().equals("Falciparum"))sim.yearlyMalariaCasesFalciparum++;
          if(plasmodium.getType().equals("Vivax"))sim.yearlyMalariaCasesVivax++;

          household.increaseWeeklyMalariaCases();
          if(plasmodium.getType().equals("Falciparum"))household.increaseNumFalciparumCases();

          if(plasmodium.getType().equals("Vivax"))household.increaseNumVivaxCases();
       }

       if(timeToCommunicability == infectionTimer)hInfectious = true;

       if(timeToRecover == infectionTimer)recover(state);

   }

   //====================================================
   public void recover(SimState state)
   {
       hInfected              = false;
       hInfectious            = false;
       treated                = false;
       susceptible            = true;

       timeToSymptoms         = -100;
       timeToRecover          = -100;
       timeToCommunicability  = -100;
       infectionTimer         = -100;

       return;

   }

   //====================================================
   public void malariaInOneYeaToHousehold(SimState state)
   {
       Bag people = household.getPeople();
       people.shuffle(state.random);

       int stats = 1;
       //System.out.println ("Num people: " + people.size()); 
       for(int i = 0; i < people.size(); i++)
       {
           Human human = ((Human)people.objs[i]);
           if(!human.getMalariaInOneYear() && stats <= numMaxProtection)
           {
              human.setMIOYOn();
              stats++;
           }
       }
   }

   //====================================================
   public int getNM(SimState state)
   {
       Bag people = household.getPeople();
       people.shuffle(state.random);

       int stats = 10000;
       //System.out.println ("Num people: " + people.size()); 
       for(int i = 0; i < people.size(); i++)
       {
           Human human = ((Human)people.objs[i]);
           if(human.getMalariaInOneYear())
           {
              stats++;
           }
       }
       return stats;
   }

   //====================================================
   public int getP(SimState state)
   {
       Bag people = household.getPeople();
       people.shuffle(state.random);
       return people.size();

   }

   //====================================================
   public boolean setAsymptomaticity(SimState state)
   {
       //Extract asymptomatic humans -----------
       double ran = state.random.nextDouble();

       if(sim.fractHumanAsymptomaticF > 0.25)
       {
          System.out.println ("Fract asymptomatic Falciparum grater thant 25%");
          System.out.println ("Not possible with this code! Sorry");
          System.exit(0);
       }

       if(sim.fractHumanAsymptomaticV > 0.25)
       {
          System.out.println ("Fract asymptomatic Vivax grater thant 25%");
          System.out.println ("Not possible with this code! Sorry");
          System.exit(0);
       }

       //low asymptomaticity for both
       if(sim.fractHumanAsymptomaticF == 0.0 && sim.fractHumanAsymptomaticV == 0.0)
       {
             ran = state.random.nextDouble();

             if(ran <= sim.fractHumanAsymptomatic)
             {
                asymptomatic = 1;
                hInfectious = true;
                susceptible = false;
      
                ran = state.random.nextDouble();
                if(ran > 0.5)plasmodium = sim.Falciparum;
                else plasmodium = sim.Vivax;
      
                return true;
             }
             else return false;
       }
       else
       {
          if(sim.asymptoSuper == 1)
          {

                 //asymptomaticy to both plasmodia alowed
          

                 boolean asy = false;
                 int stats = 0;
                 //Vivax
                 ran = state.random.nextDouble();
                 if(ran <= sim.fractHumanAsymptomaticV)
                 {
                    asymptomatic = 1;
          
                    hInfectious = true;
                    susceptible = false;
                    plasmodium = sim.Vivax;
             
                    asy = true;
                    stats++;
                 }
      
               
                 //Falciparum
                 ran = state.random.nextDouble();
                 if(ran <= sim.fractHumanAsymptomaticF)
                 {
                    asymptomatic = 1;
          
                    hInfectious = true;
                    susceptible = false;
                    plasmodium = sim.Falciparum;
             
                    asy = true;
                    stats++;
                 }

                 if(stats == 2)plasmodium = sim.bothPlasmodia;
                 return asy;
          }
          else
          {
   
                 //asymptomaticy to both plasmodia not alowed
                 ran = state.random.nextDouble();
                 if(ran >= 0.5)
                 {
                    //Vivax
                    ran = state.random.nextDouble();
                    if(ran <= sim.fractHumanAsymptomaticV * 2)
                    {
                       asymptomatic = 1;
          
                       hInfectious = true;
                       susceptible = false;
                       plasmodium = sim.Vivax;
             
                       return true;
                    }
      
                 
                 }
                 else
                 {
                    //Falciparum
                    ran = state.random.nextDouble();
                    if(ran <= sim.fractHumanAsymptomaticF * 2)
                    {
                       asymptomatic = 1;
          
                       hInfectious = true;
                       susceptible = false;
                       plasmodium = sim.Falciparum;
             
                       return true;
                    }
      
       
                 
                 }
                 return false;
             }
   

       
       }

   }


   //====================================================
   public void setupRecurrence(SimState state)
   {
       Mosquito mosquito = null;
       for(int i = 0; i < sim.mosquito_bag.size(); i++)
       {
           mosquito = ((Mosquito)sim.mosquito_bag.objs[i]);
           if(mosquito.getPlasmodium() != null)
           {
              if(mosquito.getPlasmodium().getType().equals("Vivax"))break;
           }

       }

       if(mosquito == null)
       {
          plasmodium = sim.Vivax; 
       }

       setupInfectionAfterBite(state, mosquito);

       vivaxRecurrence = false;
       vivaxRecurrenceTime = -100;

   }


   //===========4000pli=========================================
   public void pr()
   {
       System.out.println ("-----------------------------");
       //System.out.println (this.getAge() + "  mosquito age");
       //if(infectious)
       //{
       //   System.out.println (infectious);
       //}



   }


}
