/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;

import sim.field.grid.*;

import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Envelope;

import sim.util.geo.MasonGeometry;

public class InitializeLayers implements Steppable
{
   private static final long serialVersionUID = 1L;

   //====================================================
   public InitializeLayers(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;
   }

   //====================================================
   public void step(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;
   }

   //====================================================
   public void initAll(final SimState state)
   {
      System.out.println("================================================");
      System.out.println("Initialize cover pixels grid ...................");
      System.out.println(" ");

      final AmaSim sim = (AmaSim)state;

      for(int i = 0; i < sim.geoGridWidth; i++)
      {
         for(int j = 0; j < sim.geoGridHeight; j++)
         {
             CoverPixel cp = new CoverPixel(state, i, j);
             cp.setType("geo");

             //The square that corresponds to the grod coords i, j 
             //in the geo reference siystyem is built
             Point center = sim.utils.getCellCentralPoint(i, j, "geo");
             Polygon square = sim.utils.getSquareAroundPoint(center, sim.geoCellSize);

             //The coverPixel gets (i, j) as grid coordinates and
             //the correspondingi suqare in the geo ref system 
             //as masonGeoemtry
             MasonGeometry mg = new MasonGeometry(square);
             cp.setMasonGeometry(mg);

             //System.out.println (i + " " + (-j + sim.geoGridHeight - 1));
             sim.geoGrid.set(sim.utils.getCoordsToDisplay(state, i, j, "geo")[0],
                               sim.utils.getCoordsToDisplay(state, i, j, "geo")[1],
                               cp
                              );
         }
      }
   }

   //====================================================
   public void setWater(final SimState state)
   {
      System.out.println("Setting Pixels water state......................");
      System.out.println(" ");

      final AmaSim sim = (AmaSim)state;

      double totNumCells = sim.geoGridWidth * sim.geoGridHeight;

      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();
      int statsHigh = 0;
      int statsLow  = 0;
      int statsExcluded  = 0;

geo:
      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);

         if(setWaterHigh(state, cp)) statsHigh++;
         if(setWaterLow(state, cp)) statsLow++;
         if(setIsRiverExcluded(state, cp)) statsExcluded++;

         //if(cp.getIsWater() == 1)cp.setIsProductive(true);


      }

      double perc = Math.round((100 * statsHigh/totNumCells) * 100  );
      System.out.println(perc/100  + "% of the cells is covered by water in the High Water regime");

      perc = Math.round((100 * statsLow/totNumCells) * 100  );
      System.out.println(perc/100  + "% of the cells is covered by water in the Low Water regime");

      perc = Math.round((100 * statsExcluded/totNumCells) * 100  );
      System.out.println(perc/100  + "% of tthe cells is excluded from mos prod because in the middle of the river");
      System.out.println(" ");


      //System.exit(0);


   }



   //====================================================
   public boolean setWaterHigh(final SimState state, CoverPixel cp)
   {
      final AmaSim sim = (AmaSim)state;

      MasonGeometry mg = cp.getMasonGeometry();
      Polygon square = (Polygon)mg.getGeometry();

      int num_waters = sim.high_water.getGeometries().numObjs;

water:
      for (int ii = 0; ii < num_waters; ii++)
      {
         MasonGeometry mgWater = (MasonGeometry)sim.high_water.getGeometries().objs[ii];
         Polygon pwater = (Polygon)mgWater.getGeometry();

         String what = mgWater.getStringAttribute("CUBIERTA");
         if(!what.equals("AGUA")){continue;}

         //if(pwater.covers(square))
         if(pwater.covers(square) || pwater.overlaps(square))
         {
             cp.setIsWaterHigh(1); 
             cp.setIsWater(1); 
             return true;
             //System.out.println("Is Water High!!!!!");
         }
      }
  
      return false;

   }

   //====================================================
   public boolean setWaterLow(final SimState state, CoverPixel cp)
   {
      final AmaSim sim = (AmaSim)state;

      MasonGeometry mg = cp.getMasonGeometry();
      Polygon square = (Polygon)mg.getGeometry();

      int num_waters = sim.low_water.getGeometries().numObjs;

water:
      for (int ii = 0; ii < num_waters; ii++)
      {
         MasonGeometry mgWater = (MasonGeometry)sim.low_water.getGeometries().objs[ii];
         Polygon pwater = (Polygon)mgWater.getGeometry();

         //if(pwater.covers(square))
         if(pwater.covers(square) || pwater.overlaps(square))
         {
             cp.setIsWater(1); 
             cp.setIsWaterLow(1); 
             //System.out.println("Is Water Low!!!!!");
             return true;
         }
       }

  
      return false;

   }

   //====================================================
   public boolean setIsRiverExcluded(final SimState state, CoverPixel cp)
   {
      final AmaSim sim = (AmaSim)state;

      MasonGeometry mg = cp.getMasonGeometry();
      Polygon square = (Polygon)mg.getGeometry();

      int num_waters = sim.rioCutted_10mBuffer.getGeometries().numObjs;

water:
      for (int ii = 0; ii < num_waters; ii++)
      {
         MasonGeometry mgWater = (MasonGeometry)sim.low_water.getGeometries().objs[ii];
         Polygon pwater = (Polygon)mgWater.getGeometry();

         //if(pwater.covers(square))
         if(pwater.covers(square))
         {
             cp.setIsRiverExcluded(1); 
             //System.out.println("Is Water Low!!!!!");
             return true;
         }
       }
      return false;
   }



   //====================================================
   public void findDryHigh(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;
      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();

      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);

         if(cp.getIsWaterLow() == 1)
         {
            cp.setIsWaterHigh(1);    
            cp.setIsWater(1);    
         }

         if(cp.getIsWaterHigh() == 0 && cp.getIsWaterLow() == 0)
            cp.setIsDryHigh(1);
      }


   }


   //====================================================
   public void findWaterMinMaxElevation(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;

      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();

      double maxElev = -100000;
      double minElev =  100000;
      double elevation = 0.0;

      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);

         if(cp.getIsWaterHigh() == 0 && cp.getIsWaterLow() == 0)continue;

         elevation = cp.getElevation();

         if(maxElev <= elevation){maxElev = elevation;}
         if(minElev >= elevation){minElev = elevation;}
         //System.out.println("Elevation: " + elevation);
      }
      System.out.println("Max Elevation Water High: " + maxElev);
      System.out.println("Min Elevation Water Highn: " + minElev);

      sim.maxElevationHighWater = maxElev;
      sim.minElevationHighWater = minElev;

      //System.exit(0);

   }


   //====================================================
   public void setElevation(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;

      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();
      System.out.println("Num Pixels: " + num_pixels);

      //Envelope mbr = sim.elevation.getMBR();
      //System.out.println("elve mbr max Y: " + mbr.getMaxY());
      //mbr = sim.high_water.getMBR();
      //System.out.println("water mbr max Y: " + mbr.getMaxY());

      //System.out.println("elev grid width: " + sim.elevationGrid.getWidth());
      //System.out.println("elev grid height: " + sim.elevationGrid.getHeight());
      //System.exit(0);

      double maxElev = -100000;
      double minElev =  100000;

      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);
         Point point = sim.utils.getCellCentralPoint(cp.getXcor(), cp.getYcor(), "geo");

         double elevation = sim.blinearInterpolator.getValue(point.getX(), point.getY()); 

         cp.setElevation(elevation);
         if(maxElev <= elevation){maxElev = elevation;}
         if(minElev >= elevation){minElev = elevation;}
         //System.out.println("Elevation: " + elevation);
      }
      System.out.println("Max Elevation: " + maxElev);
      System.out.println("Min Elevation: " + minElev);

      //System.exit(0);
   }

   //====================================================
   public void checkLayers(final SimState state)
   {
      System.out.println("Checking Pixels water state......................");
      System.out.println(" ");

      final AmaSim sim = (AmaSim)state;

      double totNumCells = sim.geoGridWidth * sim.geoGridHeight;

      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();
      int statsRiverHigh = 0;
      int statsRiverLow  = 0;

      int statsHigh = 0;
      int statsLow  = 0;

      int numCellsStats  = 0;

      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);

         if(cp.getIsWater() == 1)cp.setIsProductive(true);

         numCellsStats++;

         if(cp.getIsWaterHigh() == 1) statsHigh++;
         if(cp.getIsWaterLow() == 1) statsLow++;

      }

      System.out.println("Num Cells count: "  + numCellsStats);
      System.out.println(" ");

      System.out.println("Water high: "  + statsHigh);
      System.out.println(" ");

      System.out.println("Water Low: "  + statsLow);
      System.out.println(" ");

      System.out.println(".................................................");
      //System.exit(0);

   }



}
