/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;

public class LayersDynamics implements Steppable
{
   private static final long serialVersionUID = 1L;

   //====================================================
   public Stoppable stopper;

   //====================================================
   public LayersDynamics(final SimState state)
   {
      System.out.println("================================================");
      System.out.println("Initialize Layers Dynamics   ...................");
      System.out.println(" ");

      final AmaSim sim = (AmaSim)state;

      this.stopper = sim.schedule.scheduleRepeating(this);

      changeWaterCover(state, sim.startDate);
      //System.exit(0);

   }
   
   //====================================================
   public void step(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;

      int nowGen = (int)state.schedule.getTime();  
      int now = sim.utils.getTime();

      if(nowGen % 24 == 0)
      {
          generateMosquitosFromHighWater(state);
      }

      if(nowGen % 96  == 0)
      {
          MeteoData md = sim.meteoData.get(now);
          String date = md.getDate();
          changeWaterCover(state, date);


      }

   }

   //====================================================
   public void changeWaterCover(final SimState state, String date)
   {
      final AmaSim sim = (AmaSim)state;

      double riverLevel = sim.dailyNanayLevel.get(date);

      riverLevel = riverLevel * (sim.maxNanayLevel - sim.minNanayLevel) + sim.minNanayLevel;
      //System.out.println("River level: " + riverLevel);

      riverLevel = riverLevel - sim.minNanayLevel;

      riverLevel = riverLevel + sim.minRiver;

      if(sim.equilibPeriod == 2184)
      {
         //System.exit(0);
         //now = 23;
         double now = (double)state.schedule.getTime();  
         if(now < sim.equilibPeriod && now > 840)
         {
             riverLevel = 86.0 + ((now - 840) / (sim.equilibPeriod - 840)) * (90.0 - 86.0);
             //System.out.println("now: " + now);
             //System.out.println("sim: " + sim.equilibPeriod);
             //System.out.println("here!!!!!!!!!!!");
         }
      }

      System.out.println("River level used: " + riverLevel);
      changeIt(state, riverLevel);
      //System.exit(0);
   }

   //====================================================
   public void changeIt(final SimState state, double riverLevel)
   {
      //System.out.println("Change it --------------");

      final AmaSim sim = (AmaSim)state;
      int stats = 0;

      Bag cps = sim.geoGrid.elements();
      int num_pixels = cps.size();

      for(int i =  0; i < num_pixels; i++)
      {
         CoverPixel cp = ((CoverPixel)cps.objs[i]);

         if(cp.getIsWaterLow() == 1)continue;
         if(cp.getIsWaterHigh() == 0)continue;

         if(cp.getElevation() > riverLevel)
         {
            cp.setIsWater(0); 
            //System.out.println("Dry!!");
         }
         else
         {
            cp.setIsWater(1); 
            //System.out.println("Water!!");
         }

         //Buffer area around house to test effectiveness of water 
         //habitats management
         if(cp.getIsBufferNoMos())cp.setIsWater(0); 

         //Patch for low lands -------------------------
         //if(riverLevel < 89)
         if(riverLevel < sim.riverPatch)
         {
            if(cp.getElevation() < 85) 
            {
               cp.setIsWater(0); 
            }
         }
      }
   }

   //====================================================
   public void generateMosquitosFromHighWater(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;
      CoverPixel cp;

      Bag pixels = sim.geoGrid.elements();
      pixels.shuffle(state.random);
      int len = pixels.size();
      double rest = 0;
      double dnumMos = 0;
      int numMos = 0;
      int stats = 0;

      sim.numProductivePixels = 0;

      for(int i = 0; i < len; i++)
      {
         cp = ((CoverPixel)pixels.objs[i]);

         dnumMos = cp.eggsOvoposited.get(0);

         //Reset the 0 value of ovopisited engss
         int nEggs = cp.getNEggs();
         cp.setEggsOvoposited(nEggs);
         cp.setNEggs(0);

         if(!cp.getIsProductive())continue;
         if(cp.getIsWaterHigh() == 0) continue;
         if(cp.getIsWater() == 0) continue;
         if(cp.getIsRiverExcluded() == 1) continue;

         sim.numProductivePixels++;

         if(dnumMos == 0)continue;

         dnumMos = dnumMos * sim.propOvopositionEmergentAdults;

         if(dnumMos > sim.numMosPerHighWaterArea * sim.geoCellSize * sim.geoCellSize )
         {
            dnumMos = sim.numMosPerHighWaterArea * sim.geoCellSize * sim.geoCellSize;
         }

         //System.out.println("cell area: " + sim.geoCellSize * sim.geoCellSize);
         //System.exit(0);

         if(cp.getIsWaterLow() == 1) dnumMos = dnumMos / sim. numMosHighLowWaterRatio;
         
         if(dnumMos < 1)
         {
            rest = cp.getMosRest();
            rest = rest + dnumMos;

            if(rest >= 1)
            {
               //System.out.println("Inside rest " + rest);
               numMos = (int)Math.round(rest);      
               //System.out.println("Inside numMos " + numMos);
               cp.setMosRest(0.0);
            }
            else 
            {
               cp.setMosRest(rest);
               continue;
            }
         }
         else
         {
            numMos = (int)Math.round(dnumMos);      
         }

         if(sim.maxAdultEmergent < numMos)sim.maxAdultEmergent = numMos;


         for(int m = 0; m < numMos; m++)
         {
             //System.out.println("----------------------");
             //System.out.println("Mosquito");
             sim.mosGen.generateMosquito(state, cp);
             stats++;
         }

      }
      //System.out.println(stats + " Mosquitos generated");

   }

//============================================================   
}
