/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;
import java.util.*; 


//----------------------------------------------------
public class Mosquito extends Animal 
{
   private static final long serialVersionUID = 1L;

   AmaSim sim = null;

   int restingAfterBite = -100;
   int restingAfterLayEggs = -100;
   boolean bloodFeeded = false;
   
   double sporogonic = 0.0;
   boolean sporogonicEnded;

   int matingTime = -100;

   int DD = 0;
   double incubationMinTemp = 0.0;

   String stopped = "no";

   boolean beforeMating           = true;
   public  boolean getBeforeMating(){return beforeMating;};

   boolean hostSearch             = false;
   public  boolean getHostSearch(){return hostSearch;};

   boolean breedingHabitatSearch  = false;
   public  boolean getBreedingHabitatSearch(){return breedingHabitatSearch;};

   boolean stepEnd = false;

   double timeToGetBlood = 0.0;
   double timeToLayEggs = 0.0;
   int eggDevelopmentTime = 0;

   boolean noNulliparous = false;
   public  boolean getNoNulliparous(){return noNulliparous;};

   public CoverPixel cpPosition = null;
   public void setCpPosition(CoverPixel mg){cpPosition = mg;};
   public CoverPixel getCpPosition(){return cpPosition;};

   public Boolean justArrived = true;

   //boolean tagged = false;

   //====================================================
   public Mosquito(SimState state, CoverPixel cpm, String grid)
   {

       super(state, cpm);
       cpPosition = cpm;
       sim = (AmaSim)state;

       if(!grid.equals("mosquito"))
       {
           System.out.println ("The mosquitos acept only mosquitos grid coordinates");
           System.exit(0);
       }

       this.setSpecies("Anopheles Darlingi");

       this.stopper = sim.schedule.scheduleRepeating(this);

       sim.mosquitoGrid.setObjectLocation(this, 
                     sim.utils.getCoordsToDisplay(state, cpPosition.getXcor(), cpPosition.getYcor(), "mosquito")[0],
                     sim.utils.getCoordsToDisplay(state, cpPosition.getXcor(), cpPosition.getYcor(), "mosquito")[1]
           );
       sim.mosquito_bag.add(this);

       //Select the hour of the mating ------
       selectMatingTime(state);       
       beforeMating = true;

       sporogonicEnded = false;

       eggDevelopmentTime = -100;

       noNulliparous = false;

   }

   //====================================================
   public void step(SimState state)
   {
      super.step(state);
      sim = (AmaSim)state;
      stepEnd = false;

      if(sim.mosquito_bag.size() > 1000000)
      {
        System.out.println ("Too many mosquitoes in the sim > 1000,000");
        System.exit(0);
      }


      //See if the mosquito survives this hour ----------
      ifIHaveToDie(state);
      if(stepEnd) return;


      //Infection dynamics not dep on hourly activites --
      infectionDynamics(state);

      //Hourly activities no depending on infection -----

      //Upgrade and watch timers-------------------------
      timers(state);
      if(stepEnd) return;

      if(isDayTime(state)) return;

      if(beforeMating)    searchPartner(state);
      else if(hostSearch)  searchHost(state);
      else if(breedingHabitatSearch) searchBreedingHabitat(state);
      if(sim.debug)checkLogic(state); //switch off during production
      if(stepEnd) return;


   }
   
   //====================================================
   public void timers(SimState state)
   {
      restingAfterBite--;
      restingAfterLayEggs--;
      timeToGetBlood++;
      timeToLayEggs++;
      eggDevelopmentTime--;

      //See if some timers is set-----------------------
      if(restingAfterBite    >= 0
      || restingAfterLayEggs >= 0 )stepEnd = true;;
      return;
   }

   //====================================================
   public void searchPartner(SimState state)
   {
       //If recently emerge from breeding site wait for mating
       //and move as in the blood seeking mode
       move(state, "hostSearch");
       matingTime--;
       if(matingTime == 0)
       {
          beforeMating = false;
          hostSearch   = true;
          return;
       }
   }


   //====================================================
   public void searchHost(SimState state)
   {
       //if(infectious)
       //{
       //    System.out.println ("Infectious search host !!!!");
       //    System.exit(0);
       //}
       //System.out.println ("Search host !!!!");

       //If is biting time mosquito search a human to bite
       boolean founded = cpPosition.getContainsHousehold();
       if(founded)
       {
           if(justArrived)
           {
              justArrived = false;
              return;
           }
       }

       if(isBitingTime(state) && founded)
       {
           justArrived = true;
           boolean feeded = searchSomeoneToBite(state);
           if(feeded)
           {
              hostSearch = false;
              breedingHabitatSearch = true;
              restingAfterBite = sim.restingTimeAfterBite;
              eggDevelopmentTime = sim.eggDevelopmentTime;
              timeToLayEggs = 0.0;
           }

           stepEnd = true; 
           return;
       }

       //Not biting time -> wait for the darkness
       if(founded)
       {
           stepEnd = true; 
           return;
       }

       //Not near to humans -> move
       move(state, "hostSearch");
   }

   //====================================================
   public void xsearchBreedingHabitat(SimState state)
   {
       CoverPixel gcp = cpPosition.getContainsBreeding(state);
       //System.out.println ("Search for breeding habitat fuori");

       if(gcp != null && eggDevelopmentTime < 0)
       {
           //System.out.println ("Search for breeding habitat dentro");
           layEggs(state, gcp); 
           breedingHabitatSearch = false;
           hostSearch            = true;


           //if(infectious)System.out.println ("Infectious founded breeding  !!!!");
           stepEnd = true; 
           return;
       }

       //Not near to breeding site -> move
       move(state, "breedingSiteSearch");
   }

   //====================================================
   public void searchBreedingHabitat(SimState state)
   {
       CoverPixel gcp = cpPosition.getContainsBreeding(state);
       //System.out.println ("Search for breeding habitat fuori");

       if(gcp != null)
       {
           if(justArrived)
           {
              justArrived = false;
              return;
           }

           if(eggDevelopmentTime < 0)
           {
               //System.out.println ("Search for breeding habitat dentro");
               layEggs(state, gcp); 
               breedingHabitatSearch = false;
               hostSearch            = true;

               //if(infectious)System.out.println ("Infectious founded breeding  !!!!");
               stepEnd = true; 
               justArrived = true;
               return;
           }
       }
       else
       {
           //Not near to breeding site -> move
           move(state, "breedingSiteSearch");
       }

   }

   //====================================================
   public void infectionDynamics(SimState state)
   {
       //Upgrade the degree days if infected
       if(infected > 0)
       {
           upgradeDegreesDays(state);
           //Check if the incubation period is finish
           if(infected <= 0)
           {
              //System.out.println ("Mosquito became infectious !!!!!!");
              this.setInfectious(true);
              sporogonicEnded = true;
              infected = -1;
           }
       }
   }

   //====================================================
   public boolean searchSomeoneToBite(SimState state)
   {
      boolean feeded = false;

      Household hh = selectHousehold(state);
      if(hh == null) return feeded; 
      Human h = selectHumanInHousehold2(state, hh);

      if(h != null){ feeded =  bite(state, h); }
      return feeded;
   }

   //====================================================
   public CoverPixel giveNextPixel(SimState state, String what)
   {
       sim = (AmaSim)state;
       CoverPixel tcp;

       List<Double> weightsList = new ArrayList<Double>();

       //convert from mosquito to geo grid coordinates --
       //System.out.println ("------------------------");
       Bag neig = cpPosition.getMooreNeighbors(state, "doShuffle", "mosquito");

       for(int i =  0; i < neig.size(); i++)
       {
           tcp = (CoverPixel)neig.objs[i];
       
           if(what == "hostSearch")
           {
             if(tcp.getContainsHousehold())weightsList.add(sim.randomWalkWeight);
             else weightsList.add(1 - sim.randomWalkWeight);
           }
           else if(what == "breedingSiteSearch")
           {
             if(tcp.getContainsBreeding(state) != null)weightsList.add(sim.randomWalkWeight);
             else weightsList.add(1 - sim.randomWalkWeight);
           }
           else
           {
              System.out.println ("Not understand what is what mosquito");
              System.exit(0);
           }
       }


       double sum = 0.0;
       for(int i =  0; i < weightsList.size(); i++)
       {
           sum = sum + weightsList.get(i);
       }
       for(int i =  0; i < weightsList.size(); i++)
       {
           weightsList.set(i, weightsList.get(i));
       }

       sum = 0.0;
       for(int i =  0; i < neig.size(); i++)
       {
           tcp = (CoverPixel)neig.objs[i];
           sum = sum + weightsList.get(i);
           double ran = state.random.nextDouble();
           if(ran <= sum)return tcp;
       }

       return cpPosition;
   
   }

   //====================================================
   public void move(SimState state, String what)
   {
       sim = (AmaSim)state;

       cpPosition = giveNextPixel(state, what);

       sim.mosquitoGrid.setObjectLocation(this, 
                     sim.utils.getCoordsToDisplay(state, cpPosition.getXcor(), cpPosition.getYcor(), "mosquito")[0],
                     sim.utils.getCoordsToDisplay(state, cpPosition.getXcor(), cpPosition.getYcor(), "mosquito")[1]
           );


       if(sim.debug)
       {
          if(stopped.equals("si"))
          {
              System.out.println ("stopped!!!!!!!!!!!!!!!!!");
          }
       }


   }


   //====================================================
   public Household selectHousehold(SimState state)
   {

      Bag households = cpPosition.getHouseholds();
      households.shuffle(state.random);

      int num_houses = households.size();


      if(num_houses > 0)
      {
         //select an household in the pixel wehre it is
         int ri = state.random.nextInt(num_houses);
         Household household = ((Household)households.objs[ri]);

         return household;

         //System.out.println (household.getFractInfectedPeople());
      }

      return null;
   }

   //====================================================
   public Human selectHumanInHousehold(SimState state, Household hh)
   {
      sim = (AmaSim)state;
      //select a person inside the selected household
      Bag hpeople = hh.getPeople();
      hpeople.shuffle(state.random);

      int num_people = hpeople.size();
      double Psum = 0.0;
      int Iage = 0; 
      double Yage = 0.0; 
      double p = 0.0;

      for(int i =  0; i < num_people; i++)
      {
         Human h = ((Human)hpeople.objs[i]);
         Iage = h.getAge();
         Yage = Iage /(365 * 24);
         p = (1 - sim.cAgeDepForceOfInfection * Math.exp(- Yage * sim.kAgeDepForceOfInfection) );

         Psum = Psum + p;
      }


      double ran = state.random.nextDouble();


      double dsup = 0.0;
      double dinf = 0.0;
      Human h = ((Human)hpeople.objs[0]);
      for(int i =  0; i < num_people; i++)
      {
         h = ((Human)hpeople.objs[i]);
         Iage = h.getAge();
         Yage = Iage /(365 * 24);
         p = (1 - sim.cAgeDepForceOfInfection * Math.exp(- Yage * sim.kAgeDepForceOfInfection) );
         p = p / Psum;
         dsup = dsup + p;

         if( p >= dinf  && p < dsup )return h;

         dinf = dinf + p;
      }


      return h;
   }

   //====================================================
   public boolean bite(SimState state, Human human)
   {
       sim = (AmaSim)state;
  
       double ran = state.random.nextDouble();
       if(ran <= sim.probFailBite) return false;

       if(human.getIsProtected()) return false;

       //System.out.println ("Inside bite  ========================");
       if(human.getMalariaInOneYear())
       {
          ran = state.random.nextDouble();
          //System.out.println ("fail!!!!!!!!-------");
          //System.out.println ("Num Mlaria in one year: " + human.getNM(state));
          //System.out.println ("Num people: " + human.getP(state));
          double prob = human.getProbFail();
          //System.out.println ("Prob: " + prob);
          if(ran <= prob)
          {
             sim.weeklyNumFailBite++;
             return false;
          }
       }

       //System.out.println ("Biting!----------------------");
       //System.out.println ("Num Mlaria in one year: " + human.getNM(state));
       //System.out.println ("Num people: " + human.getP(state));
       //System.out.println ("Human ident: " + human.getIdentity());
       //Household house = human.getHousehold();
       //System.out.println ("House ident: " + house.getIdentity());

       bloodFeeded = true;

       //Upgrade Statistics 
       sim.totalNumBite++;
       sim.weeklyNumBite++;
       sim.totNumDailyBites++;

       int now = (int)state.schedule.getTime();  
       now = (int)now % 24;
       sim.bitingHours[now]++;

       if(noNulliparous)
       {
          sim.timeToGetBlood = sim.timeToGetBlood + timeToGetBlood;
          sim.numTimeToGetBlood++;

          sim.weeklyTimeToGetBlood = sim.weeklyTimeToGetBlood + timeToGetBlood;
          sim.weeklyTimeToGetBloodCounter++;
          //System.out.println ("Time to get blood: " + timeToGetBlood);
          //timeToGetBlood = sim.weeklyTimeToGetBlood / sim.weeklyTimeToGetBloodCounter;
          //System.out.println ("Time to get blood avg: " + timeToGetBlood);
       }
       //System.out.println (now + " " + sim.bitingHours[now]);

       ran = state.random.nextDouble();

       //Mosquito is infectious------------------------
       if(human.getSusceptible() && this.getInfectious() )
       {
          //System.out.println ("Human susceptible and mis infectiuos");
          if(sim.debug)
          {
             if(this.getPlasmodium() == null)
             {
                System.out.println ("Infectiuos Mosquito with null Plasmodium");
                System.exit(0);
             }
          }

          if(human.getAsymptomatic() == 1)return true;

          if(this.getPlasmodium().getTEfficiencyMosquitoToHuman() >= ran)
          {
             human.setupInfectionAfterBite(state, this);

             sim.malariaMonthlyCases++;
             sim.weeklyNumHumanInfected++;
             //System.out.println ("Human Infected !!!!!!!!!!!!!!!!!!!");
          }
       }
       //Human is infectious--------------------------
       else if(human.getHInfectious()  && 
           this.getInfected() < 0 && !this.getInfectious())
       {
          if(sim.debug)
          {
             if(human.getPlasmodium() == null)
             {
                System.out.println ("Infectiuos Human with null Plasmodium");
                System.exit(0);
             }
          }

          Plasmodium tPlasmodium = null;
          if(human.getPlasmodium().getType().equals("bothPlasmodia"))
          {
             ran = state.random.nextDouble();
             if(ran > 0.5)tPlasmodium =  sim.Falciparum;
             else tPlasmodium = sim.Vivax;
          }
          else
          {
             tPlasmodium = human.getPlasmodium(); 
          }

          if(human.getAsymptomatic() == 1)
          {
             ran = state.random.nextDouble();
             if(tPlasmodium.getAsymptomaticTEfficiencyHumanToMosquito() >= ran)
             {
                this.setPlasmodium(tPlasmodium);
                this.setInfected(tPlasmodium.getDD());
                return true;
             }
          }
          else
          {
             ran = state.random.nextDouble();
             if(tPlasmodium.getSymptomaticTEfficiencyHumanToMosquito() >= ran)
             {
                this.setPlasmodium(tPlasmodium);
                this.setInfected(tPlasmodium.getDD());
             }
          }

       }
       return true;
   }


   //====================================================
   public boolean isBitingTime(SimState state)
   {
       boolean isTime = false;
       int now = (int)state.schedule.getTime();  
       for(int i =  - sim.bitingPeakDev; i <= sim.bitingPeakDev; i++)
       {
          int t = sim.bitingPeak + i;

          if(t < 0){t = t + 24;}
          if(t > 23){t = t - 24;}

          int day = now % 24;
          if(day == t)
          {
             //System.out.println ("biting time!! " + (now % 24 ));
             isTime = true;
             return isTime;
          }
       }
        return isTime;
   }

   //====================================================
   public void selectMatingTime(SimState state)
   {
      sim = (AmaSim)state;
      double ran = state.random.nextDouble();
      double cumD = 0.0;

      for(int i = 0; i <  sim.matingProbDist.size(); i = i + 2)
      {
         cumD = cumD + sim.matingProbDist.get(i + 1);
         if(ran < cumD)
         {
            matingTime = sim.matingProbDist.get(i).intValue();
            break;
         }
      }
   }

   //====================================================
   public boolean survive(SimState state)
   {
      int now = sim.utils.getTime();

      sim = (AmaSim)state;
      double sd = 0.0;

      MeteoData md = sim.meteoData.get(now);
      double rain = md.getRain();
      double temp = (md.getMinT() + md.getMaxT())/2;

      double expo = -(1 / (-4.4 + (1.31 * temp) - (0.03 * Math.pow(temp, 2)  )  ) );
      sd = Math.exp(expo);

      if(sim.equilibration)
      {
         double ran2 = state.random.nextDouble();
         if(ran2 > 0.7)rain = 14;
         else rain = 0;
      }

      if(rain > sim.minRainWetDay)
          sd = sd * sim.rainSurvFact;


      double ran = state.random.nextDouble();

      //System.out.println ("-----------------------------");
      //System.out.println (now);
      //System.out.println (age);
      //System.out.println (rain);
      //System.out.println ("Temp: " + temp);
      //System.out.println ("Survival prob: " + sd);
      //System.out.println (ran);
      //System.exit(0);
      if(ran <= sd)
      {
          return true;
      }
      else return false;



   }

   //====================================================
   public void die(SimState state)
   {
       sim = (AmaSim)state;
       //System.out.println (age);

       //Upgrade Statistics -----
       sim.numExistedMosquitos++;
       sim.totalLifespan = sim.totalLifespan + age;
       sim.weeklyAvgMosquitoLifeSpan = sim.weeklyAvgMosquitoLifeSpan + age;
       sim.weeklyAvgMosquitoLifeSpanCount++;

       if(sporogonicEnded)
       {
          //System.out.println ("Sporogonic: " + (double)sporogonic/24 + "  " + plasmodium.getType() + " days");
          if(plasmodium.getType().equals("Falciparum"))
          {
             sim.totalSporogonicFalciparum = sim.totalSporogonicFalciparum + sporogonic;
             sim.numSporogonicMosquitosFalciparum++;

             sim.weeklyAvgSporogonicFalciparum = sim.weeklyAvgSporogonicFalciparum + sporogonic;
             sim.weeklyAvgSporogonicFalciparumCounter++;
          
          }

          if(plasmodium.getType().equals("Vivax"))
          {
             sim.totalSporogonicVivax = sim.totalSporogonicVivax + sporogonic;
             sim.numSporogonicMosquitosVivax++;

             sim.weeklyAvgSporogonicVivax = sim.weeklyAvgSporogonicVivax + sporogonic;
             sim.weeklyAvgSporogonicVivaxCounter++;
          }

       }

       //System.out.println ("--------------------------------------");
       if(sim.mosquitoGrid.remove(this) == null)
       {
          System.out.println ("The mosquito that is stopping do not exists in the mosquitoGrid");
          System.exit(0);
       }
       sim.mosquito_bag.remove(this);

       stopped = "si";
       this.stopper.stop();         
       return;
   }


   //====================================================
   public void layEggs(SimState state, CoverPixel gcp)
   {
      //System.out.println ("Lay Eggs!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
      sim = (AmaSim)state;
      bloodFeeded = false;
      restingAfterLayEggs = sim.restingTimeAfterLayEggs;
      if(noNulliparous)
      {
         sim.timeToLayEggs = sim.timeToLayEggs + timeToLayEggs;
         sim.numTimeToLayEggs++;

         sim.weeklyTimeToLayEggs = sim.weeklyTimeToLayEggs + timeToLayEggs;
         sim.weeklyTimeToLayEggsCounter++;
      }
      timeToGetBlood = 0.0;

      noNulliparous = true;

      gcp.addEgg();
   }


   //====================================================
   public void upgradeDegreesDays(SimState state)
   {
      //System.out.println ("-----------------------------");
      sim = (AmaSim)state;

      int now = sim.utils.getTime();

      //System.out.println ("now: " + now);

      MeteoData md = sim.meteoData.get(now);
      double avgTemp = md.getMinT() + md.getMaxT();
      avgTemp = avgTemp/2;
      //System.out.println ("AvgTemp: " + avgTemp);
      //System.out.println ("infected: " + infected);

      avgTemp = avgTemp - plasmodium.getIncubationMinTemp();
      avgTemp = avgTemp/24;

      infected = infected - avgTemp;

      //System.out.println ("Incubation Temp: " + plasmodium.getIncubationMinTemp());
      //System.out.println ("AvgTemp/24: " + avgTemp);
      //System.out.println ("AvgTemp: " + avgTemp*24);
      //System.out.println ("infected: " + infected);
      //System.out.println ("ExtTime: " + infected/(avgTemp*24));
      //if(plasmodium.getType().equals("Vivax"))
      //{
      //   System.exit(0);
      //}

      sporogonic++;
   }


   //====================================================
   public void checkLogic(SimState state)
   {
      boolean error = false;

      if(beforeMating && hostSearch) error = true;
      if(beforeMating && breedingHabitatSearch) error = true;
      if(hostSearch && breedingHabitatSearch) error = true;

      if(error)
      {
         System.out.println ("Error in mosquito beahviour logic!!!!!");
         System.exit(0);
      }

   }

   //====================================================
   public void ifIHaveToDie(SimState state)
   {
      //See if hte mosquito have to die -----------------
      this.setAge(this.getAge() + 1);
      if((age % 24) == 0 && !survive(state)){die(state); stepEnd = true;}
      return;
   }



   //====================================================
   public boolean isDayTime(SimState state)
   {
       int now = (int)state.schedule.getTime();  
       now = (int)now % 24;

       if(now >= 7 && now <= 17)
       {
          
          //System.out.println ("NOOOOOOOO Move!");
          //System.out.println (now);
          //now = (int)state.schedule.getTime();  
          //System.out.println (now);
          return true; 
       }
       //System.out.println ("Move!");

       return false;
   }


   //====================================================
   public void pr()
   {
       System.out.println ("-----------------------------");
       //System.out.println (this.getAge() + "  mosquito age");
       //if(infectious)
       //{
       //   System.out.println (infectious);
       //}
   }


   //====================================================
   public Human selectHumanInHousehold2(SimState state, Household hh)
   {
      sim = (AmaSim)state;
      //select a person inside the selected household
      Bag hpeople = hh.getPeople();
      hpeople.shuffle(state.random);

      int num_people = hpeople.size();

      int ri = state.random.nextInt(num_people);

      Human h = ((Human)hpeople.objs[ri]);

      return h;
   }




}


