/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import java.io.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;


public class ReadMalariaPrevalence {
   private static final long serialVersionUID = 1L;


   String stats[] = new String[2];
   public String[] getStats() { return stats; }

   public String out_stats = "";

   public AmaSim sim = null;

   public String input_file = "";
   public void setFile(String f)
   {
       input_file = f;
       String path = "sim/app/amasim/input_data/";
       input_file = path + input_file;
   }


   //====================================================
   public ReadMalariaPrevalence(AmaSim sim)
   {

   } 


   //====================================================
   public void readLevels(AmaSim sim)
   {

       //System.out.println ("input file: " + input_file);
       //System.exit(0);
       String strLine = "";
       List<Double> ld = new ArrayList<Double>();

       try
       {
           // open the file that is the first command line parameter
           //fileinputstream fstream = new fileinputstream("sim/app/amasim/amasim.params");
           FileInputStream fstream = new FileInputStream(input_file);
           // get the object of datainputstream
           DataInputStream in = new DataInputStream(fstream);
           BufferedReader br = new BufferedReader(new InputStreamReader(in));
           //read file line by line
           while ((strLine = br.readLine()) != null)   
           {
              // print the content on the console
              //system.out.println (strline);
              //strLine = strLine.trim();
        
              if( strLine.startsWith("#"))continue;
              if( strLine.startsWith("  "))continue;

              ld = readListDouble(strLine);

              //System.out.println("-----------------------");
              //System.out.println("0: " + ld.get(0));
              //System.out.println("1: " + ld.get(1));
              
              sim.malariaPrevalence.add(ld);
                  
           }
           //close the input stream
           in.close();
        }
        catch (Exception e)
        {//catch exception if any
           System.err.println("error: " + e.getMessage());
        }

        //System.exit(0);

   }


   //Gives the daily river level=========================
   //With the same notation for date as meteo datas
   public void rewriteData(AmaSim sim)
   {

       String riverLevelStartDate = "19960101";
       String riverLevelEndDate = "19991130";

       List<String> dayList =  getDays(sim, riverLevelStartDate, riverLevelEndDate);

       System.out.println("Numbers of Malaria Prevalence days: " + dayList.size());

       double level = 0.0;
       String date = "";
       int stats = 0;

       for(int d = 0; d < dayList.size(); d++)
       { 
           level = interpolateRiverLevel(sim, d);
           date = dayList.get(d);
           //System.out.println("Date: " + date  + " Level: " + level); 
           //System.out.println(date  + " " + level); 

           sim.dailyMalariaPrevalence.put(date, level);
           stats++;
       }

       //System.out.println("Rewrite: "  + stats);
       //System.out.println("Rewrite size: "  + sim.dailyMalariaPrevalence.size());
       //System.exit(0);
   }

   //====================================================
   public double interpolateRiverLevel(AmaSim sim, Integer day)
   {
       double inter = 0.0;

       int riverLen = sim.malariaPrevalence.size();
       List<Double> rd = new ArrayList<Double>();

       int rStart = 0;

       //Determine the starting day
       for(int r = 0; r < riverLen; r++)
       { 
           rd = sim.malariaPrevalence.get(r);

           if(rd.get(0) >= day)
           {
              rStart = (r - 1);

              //System.out.println("rStart: " + rStart); 
              //System.out.println("day inside: " + day); 
              //System.out.println("rd.get(0): " + rd.get(0)); 

              break;
           }
       }

       double x0 = sim.malariaPrevalence.get(rStart).get(0);
       double x1 = sim.malariaPrevalence.get(rStart + 1).get(0);

       double fx0 = sim.malariaPrevalence.get(rStart).get(1);
       double fx1 = sim.malariaPrevalence.get(rStart + 1).get(1);

       //System.out.println("Level0: " + fx0); 
       //System.out.println("Level1: " + fx1); 

       double m = (fx1 - fx0)/(x1 - x0);

       inter = m * (day - x0) + fx0;

       return inter;

   }

   //Get the list of the of the dates where I have ======
   //the Nanay river level data =========================
   public List<String> getDays(AmaSim sim, String start, String end)
   {
       List<String> days = new ArrayList<String>();
       String date = "";
       boolean write = false;
       int stats = 0;
       String old = "";

       //start = "19960101";
       //end   = "19991130";

       int meteoLen = sim.meteoDataComplete.size();
       MeteoData md = new MeteoData(sim);

       for(int m = 0; m < meteoLen; m++)
       { 
           md = sim.meteoDataComplete.get(m);  
           date = md.getDate();

           if(date.equals(start)){write = true;}


           if(write && !date.equals(old))
           {
               stats++;
               //System.out.println((stats2 % 365) + " " + date);
               days.add(date);
               //if(stats == 366){stats2 = 0;}
           }

           if(date.equals(end)){write = false;}

           old = date;
       }

       //System.out.println(stats);

       //System.exit(0);

       return days;

   }

   
   //====================================================
   public List<Double> readListDouble(String line)
   {
       List<Double> list = new ArrayList<Double>();
       double d = 0.0;

       //System.out.println (line);

       String delims = "[ ]+";
       String[] words = line.split(delims);

       //System.out.println (words[0]);

       if(words.length < 2)
       {
          System.out.println ("Error in Malaria Prevalence data input file");
          System.out.println ("Program Exits");
          System.exit(0);
       }


       for(int i = 0; i < words.length; i++)
       { 
          try 
          {
              
             d = Double.parseDouble(words[i]);
             list.add(d);
          }
          catch (NumberFormatException e) 
          {
             System.out.println ("Error in the input Malaria Prevalence");
             System.out.println ("Program Exits");
             System.exit(0);
          }



       }


       return list;
   }

   //determines the min and man river level ----------------
   //And resize level data to the interval : [0, 1] 
   public void transformData(AmaSim sim)
   {

       String date = "";

       sim.maxMalariaPrevalence = -10000.0; 
       sim.minMalariaPrevalence = 10000.0; 
       double lev = 0.0;
       int stats = 0;

       HashMap<String, Double> tmp = new HashMap<String, Double>();

       Iterator it = sim.dailyMalariaPrevalence.entrySet().iterator();

       while (it.hasNext()) 
       { 
           Map.Entry pairs = (Map.Entry)it.next();

           lev = (Double)pairs.getValue();
           if(lev >= sim.maxMalariaPrevalence)sim.maxMalariaPrevalence = lev;
           if(lev <= sim.minMalariaPrevalence)sim.minMalariaPrevalence = lev;
           //System.out.println("date: " + (String)pairs.getKey());
           stats++;
       }

       System.out.println("Extreme Malaria Prevalence. Max: " + sim.maxMalariaPrevalence + " Min: " + sim.minMalariaPrevalence);
       //System.out.println("Num of entries: " + stats);

       it = sim.dailyMalariaPrevalence.entrySet().iterator();
       while (it.hasNext()) 
       { 
           Map.Entry pairs = (Map.Entry)it.next();

           lev = (Double)pairs.getValue();

           //lev = (lev - sim.minMalariaPrevalence)/(sim.maxMalariaPrevalence - sim.minMalariaPrevalence);

           
           tmp.put((String)pairs.getKey(), lev);

       }

       sim.dailyMalariaPrevalence = tmp;

       //it = sim.dailyNanayLevel.entrySet().iterator();
       //it = tmp.entrySet().iterator();
       //while (it.hasNext()) 
       //{ 
       //    Map.Entry pairs = (Map.Entry)it.next();

       //    lev = (Double)pairs.getValue();
       //    System.out.println("Lev: " + lev);
       //    System.out.println("date: " + (String)pairs.getKey());
       //}


   }





}//End of file
