/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;
import java.util.ArrayList;
import java.util.List;
import java.util.*; 

import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LinearRing;
import sim.util.geo.MasonGeometry;


//----------------------------------------------------
public class Utils 
{
   private static final long serialVersionUID = 1L;

   AmaSim sim = null;

   //====================================================
   public Utils(SimState state)
   {
       sim = (AmaSim)state;
   }

   //=Periodic Boundary Conditions=======================
   public Integer[] pbc(SimState state, Integer coors[], String what, String grid)
   {
       int xtmp = coors[0];
       int ytmp = coors[1];

       int height = 0;
       int width = 0;
       if(grid.equals("geo"))
       {
          height = sim.geoGridHeight;
          width  = sim.geoGridWidth;
       }
       else if(grid.equals("mosquito"))
       {
          height = sim.mosquitoGridHeight;
          width  = sim.mosquitoGridWidth;
       }
       else
       {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("pbc utils");
          System.exit(0);
       }

       if(what.equals("toroidals"))
       {
          if(xtmp < 0)              xtmp = xtmp + width;
          if(xtmp >= width)  xtmp = xtmp - width;
   
          if(ytmp < 0)              ytmp = ytmp + height;
          if(ytmp >= height) ytmp = ytmp - height;
   
          coors[0] = xtmp;
          coors[1] = ytmp;
       }
       //System.out.println ("Utils " + ytmp);

       return coors;

   }

   //=Get the integer coordinated on the discrete grid that
   //correspond to the geo coordinates of a point
   public Integer[] getDiscreteCoordinatesPoint(Point point, String grid)
   {
      Integer [] coors = {0, 0};

      int height = 0;
      int width = 0;
      if(grid.equals("geo"))
      {
         height = sim.geoGridHeight;
         width  = sim.geoGridWidth;
      }
      else if(grid.equals("mosquito"))
      {
         height = sim.mosquitoGridHeight;
         width  = sim.mosquitoGridWidth;
      }
      else
      {
         System.out.println ("Not defined grid grid type in:");
         System.out.println ("pbc utils");
         System.exit(0);
      }
         //System.out.println ("====================");


      for(int i = 0; i < width; i++)
      {
         for(int j = 0; j < height; j++)
         {
             CoverPixel cp = sim.utils.getCoverPixelFromCoords(sim, i, j, grid);
             MasonGeometry mg = cp.getMasonGeometry();
             Polygon square = (Polygon)mg.getGeometry();

             Point center = square.getCentroid();
/*
         System.out.println ("--------------------");
      System.out.println (
                point.getX() + " " +
                point.getY()
                );

      System.out.println (
                center.getX() + " " +
                center.getY()
                );

                */

             if(square.covers(point))
             {
                coors[0] = cp.getXcor(); 
                coors[1] = cp.getYcor();
                return coors;

             }

         }
      }


      System.out.println (grid);
      System.out.println ("No cell grid square contains the given point");
      System.out.println ("You got a problem here......");
      System.exit(0);
       
      return coors;
   }

   //Returns the point that corresponds to the center of a grid cell
   public Point getCellCentralPoint(int i, int j, String grid)
   {
       //System.out.println ("grid " + grid);
       Coordinate coords = null;
       if(grid.equals("geo"))
       {
          coords  = getContinuosGeoCoords(i, j);
       }
       else if(grid.equals("mosquito"))
       {
          coords  = getContinuosMosquitoCoords(i, j);
       }
       else
       {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("getCellCentralPoint utils");
          System.exit(0);
       }

       Point point = new GeometryFactory().createPoint(coords);
       return point;
   }

   //Returns the geo coordinates that correspond to 
   //the a grid point of a grid cell
   public Coordinate getContinuosGeoCoords(int i, int j)
   {
       double x = sim.geoCellSize * i + sim.geoCellSize * 0.5;
       double y = sim.geoCellSize * j + sim.geoCellSize * 0.5;

       x = x + sim.globalMBRMinX;
       y = y + sim.globalMBRMinY;

       Coordinate coords  = new Coordinate(x, y);

       return coords;
   }

   //Returns the geo coordinates that correspond to 
   //the a grid point of a grid cell
   public Coordinate getContinuosMosquitoCoords(int i, int j)
   {
       double x = sim.mosquitoCellSize * i + sim.mosquitoCellSize * 0.5;
       double y = sim.mosquitoCellSize * j + sim.mosquitoCellSize * 0.5;

       x = x + sim.globalMBRMinX;
       y = y + sim.globalMBRMinY;

       Coordinate coords  = new Coordinate(x, y);

       return coords;
   }


   //Returns the geo coordinates that correspond to 
   //the a grid point of a grid cell
   public double[] getContinuosGeopos(int i, int j)
   {
       double x = sim.geoCellSize * i + sim.geoCellSize * 0.5;
       double y = sim.geoCellSize * j + sim.geoCellSize * 0.5;

       x = x + sim.globalMBRMinX;
       y = y + sim.globalMBRMinY;

       //Coordinate coords  = new Coordinate(x, y);
       double[] coords = {x, y};

       return coords;
   }




   //Creates a square of side side around the point center
   public Polygon getSquareAroundPoint(Point center, double side)
   {
       GeometryFactory fact =  new GeometryFactory();

       double x = center.getX();
       double y = center.getY();
       double lm = side/2;

       Coordinate[] coords  =
            new Coordinate[] {
                new Coordinate(x - lm, y - lm), 
                new Coordinate(x + lm, y - lm),
                new Coordinate(x + lm, y + lm), 
                new Coordinate(x - lm, y + lm), 
                new Coordinate(x - lm, y - lm) 
            };

       LinearRing linear = new GeometryFactory().createLinearRing(coords);
       Polygon poly = new Polygon(linear, null, fact);

       return poly;
   }


   //====================================================
   public Integer[] getCoordsToDisplay(final SimState state, int i, int j, String grid)
   {
       int height = 0;
       if(grid.equals("geo"))
       {
          height = sim.geoGridHeight;
       }
       else if(grid.equals("mosquito"))
       {
          height = sim.mosquitoGridHeight;
       }
       else
       {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("getCoordsToDisplay utils");
          System.exit(0);
       }

       Integer[] coords = {0, 0};
       coords[0] = i;
       coords[1] = -j + height - 1;
       return coords;
   }

   //====================================================
   public Integer[] getCoordsFromDisplay(final SimState state, int i, int j, String grid)
   {
       int height = 0;
       if(grid.equals("geo"))
       {
          height = sim.geoGridHeight;
       }
       else if(grid.equals("mosquito"))
       {
          height = sim.mosquitoGridHeight;
       }
       else
       {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("getCoordsToDisplay utils");
          System.exit(0);
       }




       Integer[] coords = {0, 0};
       coords[0] = i;
       coords[1] = -j + height - 1;
       return coords;
   }

   //====================================================
   public CoverPixel getCoverPixelFromCoords(final SimState state, int i, int j, String grid)
   {
      //return (CoverPixel)sim.geoGrid.get(i, (j ) );
      //System.out.println (i + " " + (-j + sim.gridHeight));
      if(grid.equals("geo"))
      {
         return (CoverPixel)sim.geoGrid.get(
         getCoordsFromDisplay(state, i, j, "geo")[0], 
         getCoordsFromDisplay(state, i, j, "geo")[1] );
      }
      else if(grid.equals("mosquito"))
      {
         return (CoverPixel)sim.pixelMosquitosGrid.get(
         getCoordsFromDisplay(state, i, j, "mosquito")[0], 
         getCoordsFromDisplay(state, i, j, "mosquito")[1] );
      }
      else
      {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("getCoverPixelFromCoords utils");
          System.exit(0);

          //this only to add a return
          return (CoverPixel)sim.pixelMosquitosGrid.get(
          getCoordsFromDisplay(state, i, j, "mosquito")[0], 
          getCoordsFromDisplay(state, i, j, "mosquito")[1] );
      }
   }
/*
   public Integer[] getCellFromGeoToMosquitoGrid(SimState state, int xpos, int ypos)
   {
       //double[] ccoords = getContinuosGeopos(xpos, ypos);
       //System.out.println (coords.getX());

       Integer[] coords = {0, 0};

       double x = sim.geoCellSize * (xpos);
       double y = sim.geoCellSize * (ypos);

       coords[0] = (int)Math.floor(x/sim.mosquitoCellSize);
       coords[1] = (int)Math.floor(y/sim.mosquitoCellSize);

       //coords[0] = (int)Math.round(x/sim.mosquitoCellSize);
       //coords[1] = (int)Math.round(y/sim.mosquitoCellSize);

       return coords;
   }

   public Integer[] getCellFromMosquitoToGeoGrid(SimState state, int xpos, int ypos)
   {
       //double[] ccoords = getContinuosGeopos(xpos, ypos);
       //System.out.println (coords.getX());

       Integer[] coords = {0, 0};

       double x = sim.mosquitoCellSize * (xpos);
       double y = sim.mosquitoCellSize * (ypos);
       //System.out.println ("inside double: " + x + " "  + y);

       coords[0] = (int)Math.floor(x/sim.geoCellSize);
       coords[1] = (int)Math.floor(y/sim.geoCellSize);

       return coords;
   }

*/


   //====================================================
   public String getStringMonth(String m)
   {
       if(m.equals("01"))return "January";
       else if(m.equals("02"))return "February";
       else if(m.equals("03"))return "March";
       else if(m.equals("04"))return "April";
       else if(m.equals("05"))return "May";
       else if(m.equals("06"))return "June";
       else if(m.equals("07"))return "July";
       else if(m.equals("08"))return "August";
       else if(m.equals("09"))return "September";
       else if(m.equals("10"))return "October";
       else if(m.equals("11"))return "November";
       else if(m.equals("12"))return "December";
       else
       {
         System.out.println ("Months number betweeen 01 and 12");
         System.exit(0);
       }
       return " ";


   }

   //=Periodic Boundary Conditions=======================
   public boolean pixelIsBorder(SimState state, CoverPixel cp, String grid)
   {

       int height = 0;
       int width = 0;
       if(grid.equals("geo"))
       {
          height = sim.geoGridHeight;
          width  = sim.geoGridWidth;
       }
       else if(grid.equals("mosquito"))
       {
          height = sim.mosquitoGridHeight;
          width  = sim.mosquitoGridWidth;
       }
       else
       {
          System.out.println ("Not defined grid grid type in:");
          System.out.println ("pbc utils");
          System.exit(0);
       }


       if(cp.getXcor() == 0 || cp.getXcor() == width - 1)return true;
       if(cp.getYcor() == 0 || cp.getYcor() == height - 1)return true;

       return false;

   }


   //=Get the Right time ------------------------------
   public int getTime()
   {
      //See if it is the equilibration period
      SimState state = (SimState)sim;
      int now;

      if(sim.equilibration)
      {
         now = 23;
      }
      else
      {
         now = (int)state.schedule.getTime();  
         now = now - sim.equilibPeriod;
      }
      //System.out.println (now);
      //System.exit(0);

      return now;
   }

}


