/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;

import java.io.FileWriter;
import java.io.*;

import java.util.ArrayList;
import java.util.List;

public class WriteStats implements Steppable
{
   private static final long serialVersionUID = 1L;

   //====================================================
   public WriteStats(AmaSim sim)
   {
      File theDir = new File(sim.outPath);
      int nMax = 0;
     
      // if the directory does not exist, create it
      if (!theDir.exists())
      {
        System.out.println("creating directory: " + sim.outPath);
        theDir.mkdir();
        sim.outFileNumbering = "1"; 
      }
      else
      {
         String [] directoryContents = theDir.list();

         List<String> fileLocations = new ArrayList<String>();

         for(String fileName: directoryContents) 
         {

             if(!fileName.startsWith("output"))continue;
             //System.out.println (fileName);

             String delims = "\\.";
             String[] words = fileName.split(delims);
             if(!words[1].equals("xls"))continue;

             System.out.println (fileName);

             delims = "_";
             words = words[0].split(delims);

             int len = words.length;

             //System.out.println ("Num words: " + len);
             //System.out.println (words[0]);
             if(!words[0].equals("output"))continue;
             int n = Integer.parseInt(words[len - 1]);
             if(n >= nMax)nMax = n;

         }
         nMax++;
         sim.outFileNumbering = "" + nMax;

         if(directoryContents.length == 0)
         {
            sim.outFileNumbering = "1";
         }
      }
      //System.out.println ("Numbering: " + sim.outFileNumbering);
      

      //System.exit(0);

   }

   //====================================================
   public void step(final SimState state)
   {
       final AmaSim sim = (AmaSim)state;

       int now = (int)state.schedule.getTime();  
       if(now % 168 == 0)
       {
          try {
                 sim.writeXls.write(state);
                  
             } catch (FileNotFoundException e) {
                 e.printStackTrace();
             } catch (IOException e) {
                 e.printStackTrace();
             }
           }


       //This writes only at the and of the simulation ----
       if(now == sim.num_step - 1 )
       {
          //writeOutputs(state);
          try {
                 sim.writeXls.write(state);
                  
             } catch (FileNotFoundException e) {
                 e.printStackTrace();
             } catch (IOException e) {
                 e.printStackTrace();
             }
       }


       if(now == sim.num_step - 1 )
       {
          //writeOutputs(state);
          try {
                 sim.writeXls.writeHousehold(state);
                  
             } catch (FileNotFoundException e) {
                 e.printStackTrace();
             } catch (IOException e) {
                 e.printStackTrace();
             }
       }


      
      //System.exit(0);
   }

   //====================================================
   public void writeOutputs(SimState state)
   {
       final AmaSim sim = (AmaSim)state;
       writeArrayDouble(state, sim.monthlyMalariaIncidence, "monthlyMalariaIncidence");
       writeArrayListDouble(state, sim.malariaIncidenceSerie, "malariaIncidenceSerie");
   }

   //====================================================
   public void writeArrayListDouble(SimState state, List<Double>  arrayList, String name)
   {

      final AmaSim sim = (AmaSim)state;

      String file_name = name.concat(".dat");
      String tmp = file_name;
      file_name = "sim/app/AmaSim/outputs/";
      file_name = file_name.concat(tmp);
      String line = "";

      //System.out.println (file_name);

      File f = new File(file_name);

      if(!f.exists())
      {
         try
         {
            f.createNewFile();
         }
         catch(IOException e)
         {
            System.out.println ("Cannot create a file...");
         }
      }



      try
      {
         FileWriter file = new FileWriter(file_name, true);

         for(int i = 0; i < arrayList.size(); i++)
         {
            line = i + "  "  + arrayList.get(i) + "\n";
            file.write(line);
         }

         file.close();
      }
      catch (IOException e)
      {
         System.out.println ("exception opening file");
      }

      //System.exit(0);



   }


   //====================================================
   public void writeArrayDouble(SimState state, double[] array, String name)
   {

      final AmaSim sim = (AmaSim)state;

      String file_name = name.concat(".dat");
      String tmp = file_name;
      file_name = "sim/app/AmaSim/outputs/";
      file_name = file_name.concat(tmp);
      String line = "";

      //System.out.println (file_name);

      File f = new File(file_name);

      if(!f.exists())
      {
         try
         {
            f.createNewFile();
         }
         catch(IOException e)
         {
            System.out.println ("Cannot create a file...");
         }
      }



      try
      {
         FileWriter file = new FileWriter(file_name, true);

         for(int i = 0; i < array.length; i++)
         {
            line = i + "  "  + array[i] + "\n";
            file.write(line);
         }

         file.close();
      }
      catch (IOException e)
      {
         System.out.println ("exception opening file");
      }

      //System.exit(0);



   }


   //====================================================
   public void erase_all()
   {
       File dir =  new File("sim/app/AmaSim/outputs/");
       if(dir.isDirectory())
       {
           for(File c : dir.listFiles())
           {
              
              System.out.println ("Removing file: " + c.getName() + " from outputs directory..........");
              boolean success =  c.delete();
              //if(!success)
              //{
              //   System.out.println ("Cannot remove files from outputs directory");
              //   System.exit(0);
              //}
           
           }
       
       }
       

   }


   //====================================================
   public void write(String file_name, double x, double y)
   {
      file_name = file_name.concat(".out");
      String tmp = file_name;
      file_name = "sim/app/AmaSim/outputs/";
      file_name = file_name.concat(tmp);
      //System.out.println (file_name);

      File f = new File(file_name);

      if(!f.exists())
      {
         try
         {
            f.createNewFile();
         }
         catch(IOException e)
         {
            System.out.println ("Cannot create a file...");
         }
      }

      String line = x + "  "  + y + "\n";


      try
      {
         FileWriter file = new FileWriter(file_name, true);
         file.write(line);
         file.close();
      }
      catch (IOException e)
      {
         System.out.println ("exception opening file");
      }

      //System.exit(0);

   
   }


   //====================================================
   public void write_array(String file_name, double x)
   {
      file_name = file_name.concat(".out");
      String tmp = file_name;
      file_name = "sim/app/AmaSim/outputs/";
      file_name = file_name.concat(tmp);
      //System.out.println (file_name);

      File f = new File(file_name);

      if(!f.exists())
      {
         try
         {
            f.createNewFile();
         }
         catch(IOException e)
         {
            System.out.println ("Cannot create a file...");
         }
      }

      String line = x + "\n";


      try
      {
         FileWriter file = new FileWriter(file_name, true);
         file.write(line);
         file.close();
      }
      catch (IOException e)
      {
         System.out.println ("exception opening file");
      }

      //System.exit(0

   }


   //====================================================
   public void write_data(String file_name, double x, double y)
   {
      file_name = file_name.concat(".dat");
      String tmp = file_name;
      file_name = "sim/app/AmaSim/grace/";
      file_name = file_name.concat(tmp);
      //System.out.println (file_name);

      File f = new File(file_name);

      if(!f.exists())
      {
         try
         {
            f.createNewFile();
         }
         catch(IOException e)
         {
            System.out.println ("Cannot create a file...");
         }
      }

      String line = x + "  "  + y + "\n";


      try
      {
         FileWriter file = new FileWriter(file_name, true);
         file.write(line);
         file.close();
      }
      catch (IOException e)
      {
         System.out.println ("exception opening file");
      }

      //System.exit(0);

   
   }

   //====================================================
   public void erase_all_data()
   {
       File dir =  new File("sim/app/AmaSim/grace/");
       if(dir.isDirectory())
       {
           for(File c : dir.listFiles())
           {
              
              System.out.println ("Removing file: " + c.getName() + " from grace directory..........");
              boolean success =  c.delete();
              //if(!success)
              //{
              //   System.out.println ("Cannot remove files from outputs directory");
              //   System.exit(0);
              //}
           
           }
       
       }
       

   }




}
