/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import java.io.*;
import java.util.*;

import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.WorkbookFactory; // This is included in poi-ooxml-3.6-20091214.jar
import org.apache.poi.ss.usermodel.Workbook;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;

import java.util.logging.Level;
import java.util.logging.Logger;
import sim.engine.SimState;
import sim.util.*;

import java.util.List;
import java.util.ArrayList;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

public class WriteXlsOutput
{
    //private static final long serialVersionUID = -4554882816749973618L;
    private static final long serialVersionUID = 1L;

    public String file_name;

    //====================================================
    public WriteXlsOutput(AmaSim sim)
    {
       DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd-HH-mm");
       Date date = new Date();
       //System.out.println(dateFormat.format(date));

       String tmp = "output_" + dateFormat.format(date) + "_" + sim.outFileNumbering + ".xls";
       file_name = sim.outPath;
       file_name = file_name.concat(tmp);
      
       //System.out.println(file_name);
       //System.exit(0);
    }

    //====================================================
    public void write(SimState state) throws IOException
    {
      AmaSim sim = (AmaSim)state;
 
      //erase_file();

      System.out.println("================================================");
      System.out.println ("Writing  outputs ..............................");
      System.out.println(" ");

      FileInputStream file = new FileInputStream(file_name);

      HSSFWorkbook workbook = new HSSFWorkbook(file);
      HSSFSheet sheet;

      sheet = workbook.getSheet(sim.outDir);
      //If the sheet !exists a new sheet is created --------- 
      if(sheet == null)sheet = workbook.createSheet(sim.outDir);

      workbook.setSheetOrder(sim.outDir, 0);

      Cell cell = null;

      int lastRow = sheet.getLastRowNum();

      //System.out.println("Last row:" + lastRow);
      lastRow++;

      int size = sim.weeklyData.size();
      for(int i = 0; i < size; i++)
      {
          Row row = sheet.createRow(lastRow++);

          Object [] objArr = (Object[])sim.weeklyData.get(i);
          //System.out.println(lastRow);
          //System.out.println(objArr.length);

          int cellnum = 0;
          for (Object obj : objArr) 
          {
             //System.out.println("------------");
             //System.out.println(obj);
             cell = row.createCell(cellnum++);


             if(obj instanceof Date) 
             {
                 cell.setCellType(1);
                 cell.setCellValue((String)obj);
                 //System.out.println("Date....");
             }
             else if(obj instanceof String)
             {
                cell.setCellType(1);
                cell.setCellValue((String)obj);
                //System.out.println("String....");
             }
             else if(obj instanceof Double)
             {
                cell.setCellType(0);
                cell.setCellValue((Double)obj);
                //System.out.println("Double....");
             }
          }
      }
      //System.exit(0);

      sim.weeklyData = new ArrayList<Object[]>();
      file.close();
 
      File out_file = new File(file_name);

      FileOutputStream out = new FileOutputStream(out_file);

      workbook.write(out);
      out.close();
      System.out.println("Output data written successfully.....");
           

    }

   //====================================================
   public void erase_file()
   {

    	try{
 
    		File file = new File(file_name);
 
    		if(file.delete()){
    			System.out.println(file.getName() + " is deleted!");
    		}else{
    			System.out.println("Delete operation is failed.");
    		}
 
    	}catch(Exception e){
 
    		e.printStackTrace();
 
    	}
 
 
   }

    //====================================================
    public void writeInput(AmaSim sim)
    {
      ReadInput input = new ReadInput(sim);
      List<String> list = input.getInputList();

      System.out.println("================================================");
      System.out.println ("Writing  the input file into the output  ......");
      System.out.println(" ");

      HSSFWorkbook workbook = new HSSFWorkbook();
      HSSFSheet sheet = workbook.createSheet("Input File");
 
      int rownum = 0;
      int size = list.size();

      for(int i = 0; i < size; i++)
      {
          rownum++;
          Row row = sheet.createRow(rownum);

          String line = list.get(i);

          Cell cell = row.createCell(0);
          cell.setCellValue((String)line);

      }
 
      try {

          FileOutputStream out = 
             new FileOutputStream(new File(file_name));
          workbook.write(out);
          out.close();
          System.out.println("Excel written successfully..");
           
      } catch (FileNotFoundException e) {
          e.printStackTrace();
      } catch (IOException e) {
          e.printStackTrace();
      }

      //System.exit(0);

    }

    //====================================================
    public void writeHousehold(SimState state) throws IOException
    {
      AmaSim sim = (AmaSim)state;
 
      //erase_file();

      System.out.println("================================================");
      System.out.println ("Writing  outputs households....................");
      System.out.println(" ");

      String sheetName = "Households";

      FileInputStream file = new FileInputStream(file_name);

      HSSFWorkbook workbook = new HSSFWorkbook(file);
      HSSFSheet sheet;

      sheet = workbook.getSheet(sheetName);
      //If the sheet !exists a new sheet is created --------- 
      if(sheet == null)sheet = workbook.createSheet(sheetName);

      workbook.setSheetOrder(sheetName, 1);

      Cell cell = null;

      int lastRow = sheet.getLastRowNum();

      //System.out.println("Last row:" + lastRow);
      lastRow++;

      //Writes the name of the columns
      Row row = sheet.createRow(lastRow++);

      int cellnum = 0;

      //Id of the household
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Household Id");

      //Num people in the household
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Num people in the household");

      //household xcoord
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Household Xcoord");

      //household xcoord
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Household Ycoord");

      //household num falciparum cases during the length of the simulation
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Num. falciparum cases");

      //household num vivax cases during the length of the simulation
      cell = row.createCell(cellnum++);
      cell.setCellType(1);
      cell.setCellValue("Num. vivax cases");


      int size = sim.household_bag.size();
      for(int i = 0; i < size; i++)
      {
          row = sheet.createRow(lastRow++);

          Household hh = (Household)sim.household_bag.get(i);
          //System.out.println(lastRow);
          //System.out.println(objArr.length);

          cellnum = 0;

          //Id of the household
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getIdentity());

          //Num people in the household
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getPeople().size());

          //household xcoord
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getGeoPoint().getX());

          //household xcoord
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getGeoPoint().getY());

          //household num falciparum cases during the length of the simulation
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getNumFalciparumCases());

          //household num vivax cases during the length of the simulation
          cell = row.createCell(cellnum++);
          cell.setCellType(0);
          cell.setCellValue((double)hh.getNumVivaxCases());

      }
      //System.exit(0);

      sim.weeklyData = new ArrayList<Object[]>();
      file.close();
 
      File out_file = new File(file_name);

      FileOutputStream out = new FileOutputStream(out_file);

      workbook.write(out);
      out.close();
      System.out.println("Output data written successfully.....");
           

    }




}
