/*
  Copyright 2011 by Francesco Pizzitutti
  Licensed under the Academic Free License version 3.0
  See the file "LICENSE" for more information
*/

package sim.app.AmaSim;

import sim.engine.*;
import sim.util.*;

public class startAll implements Steppable
{
   private static final long serialVersionUID = 1L;

   public AmaSim sim = null;
   public SimState state = null;

   //====================================================
   public startAll(AmaSim psim)
   {
      sim = psim;
      sim.utils = new Utils(sim);

      System.out.println("================================================");
      System.out.println ("== startAll ===================================");
      //Print the input file content to te output--------
      ReadInput input = new ReadInput(sim);
      //input.PrintInput();
      //System.exit(0);

      //Generates the two kind of Plasmodia ----------------
      System.out.println("Reading Plasmodia data -------------------------");
      sim.Falciparum         = new Plasmodium(sim, "Falciparum");
      sim.Vivax              = new Plasmodium(sim, "Vivax");
      sim.bothPlasmodia      = new Plasmodium(sim, "bothPlasmodia");

      sim.Falciparum.init(sim, input);
      sim.Vivax.init(sim, input);
      sim.bothPlasmodia.init(sim, input);


      //System.out.println ("numMosPerHigh " + sim.numMosPerHighWaterArea);
      //System.out.println ("propOvo " + sim.propOvopositionEmergentAdults);
      //System.out.println ("HighLowRatio " + sim.numMosHighLowWaterRatio);
      //System.exit(0);

      //Read the metereological data -----------------------
      if(sim.withDates)
      {
         ReadXlsMeteo rxlm = new ReadXlsMeteo(sim);
         rxlm.read(sim);
         rxlm.checkData(sim);
         rxlm.closeHoles(sim);
         //System.exit(0);

         rxlm.rescaleData(sim);
         rxlm.transformInHourly(sim);
         rxlm.cutMeteoData(sim);
         sim.num_step = sim.meteoData.size() - 1;
         sim.num_step = sim.num_step + sim.equilibPeriod;
      }
      //System.exit(0);

      //Read the Nanay River level data--------------
      sim.readRiver.setFile("digitalized_nanay_river_level_bautista.txt");
      sim.readRiver.readLevels(sim);
      sim.readRiver.rewriteData(sim);
      sim.readRiver.transformData(sim);
      //System.exit(0);

      //Read the malaria prevalence data-------------
      sim.readMalaria.setFile("digitalized_malaria_prevalence_bautista.txt");
      sim.readMalaria.readLevels(sim);
      sim.readMalaria.rewriteData(sim);
      sim.readMalaria.transformData(sim);
      //System.exit(0);

      //Read the input geo covers----------------------------
      if(!sim.gridSetupDone)
      {
         ReadShp RShp = new ReadShp(sim);
         RShp.gridSetup(sim);
      }
      //System.exit(0);

      //Initialize the Geo grid  is important the order of calls---
      sim.initLayer = new InitializeLayers(sim);
      sim.initLayer.initAll(sim);
      sim.initLayer.setWater(sim);
      //sim.initLayer.findRiverBanks(sim);
      sim.initLayer.findDryHigh(sim);
      sim.initLayer.checkLayers(sim);

      //System.exit(0);

      //sim.initLayer.setWater(sim, "low");

      //sim.initLayer.setCover(sim);

      //Bicubic interpolator --->> not works
      //sim.bInterpolator = new BicubicInterpolator(sim);
      //Bilinear interpolator -
      sim.blinearInterpolator = new BilinearInterpolator(sim);
      sim.initLayer.setElevation(sim);
      sim.initLayer.findWaterMinMaxElevation(sim);
      //System.exit(0);

      LayersDynamics ld = new LayersDynamics(sim);
      //System.exit(0);
      

      //Initialize the Piuxel mosquito grid grid  ----------------
      sim.initPixelMosquitosGrid = new InitializePixelMosquitosGrid(sim);
      sim.initPixelMosquitosGrid.initAll(sim);
      sim.initPixelMosquitosGrid.linkTwoGrids(sim);
      //System.exit(0);

      //Generate Households----------------------------------
      sim.hhGen = new HouseholdsGenerator(sim);
      sim.hhGen.initHh(sim);
      //System.exit(0);

      //Initialize the households and Breeding insede
      //mosquitos pixel grid pixels
      sim.initPixelMosquitosGrid.setHouseholds(sim);
      //sim.initPixelMosquitosGrid.setBreeding(sim);

      //To write outputs at the end of the simulation -------
      WriteStats ws = new WriteStats(sim);
      sim.schedule.scheduleRepeating(ws);
 
      //Generate Mosquitos----------------------------------
      sim.mosGen = new MosquitosGenerator(sim);
      //sim.mosGen.initMos(sim);
      //sim.schedule.scheduleRepeating(sim.mosGen);

      sim.mosGen.generateMosquitosStartEggs(sim);

      //Start the General Statistics module -------------------
      //System.exit(0);
      if(sim.stats)
      {
          Statistic statistics = new Statistic(sim);
          //Start repeat at 1.0 with ordering 10
          sim.schedule.scheduleRepeating(1.0, 10, statistics);
      }

     //To test all =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

    //System.exit(0);


   }

   //====================================================
   public void step(final SimState state)
   {
      final AmaSim sim = (AmaSim)state;
      sim.equilibPeriodCounter++;
      if(sim.equilibPeriodCounter == sim.equilibPeriod )
      {
         sim.equilibration = false;
         //sim.hhGen.setUpOneMalaria(sim);
      }
      
      //int now = sim.utils.getTime();
      //System.out.println ("now print output = " + now);
      //now = (int)state.schedule.getTime();  
      //System.out.println ("now print output = " + now);
      //System.exit(0);

   }


//============================================================   
}
